/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/******************************************************************************/
/*  Communication with an EEPROM (e.g. 2465) via I2C bus                      */
/*  The I2C module of the MSP430F2274 is used to communicate with the EEPROM. */
/*  The "Byte Write", "Page Write", "Current Address Read", "Random Read",    */
/*  "Sequential Read" and "Acknowledge Polling" commands or the EEPROM are    */
/*  realized.                                                                 */
/*                                                                            */
/*  developed with IAR Embedded Workbench V5.20.1                             */
/*                                                                            */
/*  Texas Instruments                                                         */
/*  William Goh                                                               */
/*  March 2011                                                                */
/*----------------------------------------------------------------------------*/
/*  updates                                                                   */
/*    Jan 2005:                                                               */
/*        - updated initialization sequence                                   */
/*    March 2009:                                                             */
/*        - updated code for 2xx USCI module                                  */
/*        - added Page Write and Sequential Read functions                    */
/*    March 2011:                                                             */
/*        - Fixed Random and Sequential Reads to Restart                      */
/*        - Added Page Write to support greater than 256 bytes                */
#include <msp430.h>
#include <stdint.h>
#include "I2Croutines.h"

#if (EEPROM_ADDRESS_BYTE_SIZE == 2)

#define  MAXPAGEWRITE   64

int PtrTransmit;
uint8_t I2CBufferArray[MAXPAGEWRITE + EEPROM_ADDRESS_BYTE_SIZE];
uint8_t I2CBuffer;

/*----------------------------------------------------------------------------*/
// Description:
//   Initialization of the I2C Module
/*----------------------------------------------------------------------------*/
void InitI2C(uint8_t eeprom_i2c_address,
             uint8_t eeprom_i2c_speed)
{
    I2C_PORT_SEL |= SDA_PIN + SCL_PIN;      // Assign I2C pins to USCI_B0

    // Use this bit for write-protect enable/disable
    WP_PORT_OUT |= WP_PIN;
    WP_PORT_DIR |= WP_PIN;

    // Recommended initialisation steps of I2C module as shown in User Guide:
    UCB0CTLW0 = UCSWRST;                    // Enable SW reset
    UCB0CTLW0 |= UCMODE_3 | UCMST | UCSSEL__SMCLK; // I2C Master mode, SMCLK source
    UCB0BRW = eeprom_i2c_speed;                // fSCL = SMCLK/12 = ~100kHz
    UCB0I2CSA = eeprom_i2c_address;         // define Slave Address
                                            // In this case the Slave Address
                                            // defines the control byte that is
                                            // sent to the EEPROM.
    UCB0CTLW0 &= ~UCSWRST;                  // Clear SW reset, resume operation

    if(UCB0STAT & UCBBUSY)                  // test if bus to be free
    {                                       // otherwise a manual Clock on is
                                            // generated
        I2C_PORT_SEL &= ~SCL_PIN;           // Select Port function for SCL
        I2C_PORT_OUT &= ~SCL_PIN;           //
        I2C_PORT_DIR |= SCL_PIN;            // drive SCL low
        I2C_PORT_SEL |= SDA_PIN + SCL_PIN;  // select module function for the
                                            // used I2C pins
    }
    ;
}

/*---------------------------------------------------------------------------*/
// Description:
//   Initialization of the I2C Module for Write operation.
/*---------------------------------------------------------------------------*/
void I2CWriteInit(void)
{
    UCB0CTLW0 |= UCTR;                      // UCTR=1 => Transmit Mode (R/W bit = 0)
    UCB0IFG &= ~UCTXIFG0;
    UCB0IE &= ~UCRXIE0;                     // disable Receive ready interrupt
    UCB0IE |= UCTXIE0;                      // enable Transmit ready interrupt
}

/*----------------------------------------------------------------------------*/
// Description:
//   Initialization of the I2C Module for Read operation.
/*----------------------------------------------------------------------------*/
void I2CReadInit(void)
{
    UCB0CTLW0 &= ~UCTR;                     // UCTR=0 => Receive Mode (R/W bit = 1)
    UCB0IFG &= ~UCRXIFG0;
    UCB0IE &= ~UCTXIE0;                     // disable Transmit ready interrupt
    UCB0IE |= UCRXIE0;                      // enable Receive ready interrupt
}

/*----------------------------------------------------------------------------*/
// Description:
//   Byte Write Operation. The communication via the I2C bus with an EEPROM
//   (2465) is realized. A data byte is written into a user defined address.
/*----------------------------------------------------------------------------*/
void EEPROM_ByteWrite(uint16_t Address,
                      uint8_t Data)
{
    while(UCB0STATW & UCBUSY)
    {
        ;                                   // wait until I2C module has
    }
    // finished all operations.

    I2CBufferArray[2] = Address >> 8;       // Low byte address.
    I2CBufferArray[1] = Address & 0xFF;     // High byte address.
    I2CBufferArray[0] = Data;
    PtrTransmit = EEPROM_ADDRESS_BYTE_SIZE; // set I2CBufferArray Pointer

    DISABLE_WRITE_PROTECT

    I2CWriteInit();
    UCB0CTLW0 |= UCTXSTT;                   // start condition generation
                                            // => I2C communication is started
    __bis_SR_register(LPM0_bits + GIE);     // Enter LPM0 w/ interrupts
    __no_operation();
    UCB0CTLW0 |= UCTXSTP;                   // I2C stop condition
    while(UCB0CTLW0 & UCTXSTP)
    {
        ;                                   // Ensure stop condition got sent
    }
    ENABLE_WRITE_PROTECT
}

/*----------------------------------------------------------------------------*/
// Description:
//   Page Write Operation. The communication via the I2C bus with an EEPROM
//   (24xx65) is realized. A data byte is written into a user defined address.
/*----------------------------------------------------------------------------*/
void EEPROM_PageWrite(uint16_t StartAddress,
                      uint8_t * Data,
                      uint16_t Size)
{
    volatile uint16_t i = 0;
    volatile uint8_t counterI2cBuffer;
    uint8_t adr_hi;
    uint8_t adr_lo;
    uint16_t currentAddress = StartAddress;
    uint16_t currentSize = Size;
    uint16_t bufferPtr = 0;
    uint8_t moreDataToRead = 1;

    while(UCB0STATW & UCBUSY)
    {
        ;                                     // wait until I2C module has
    }
    // finished all operations.

    DISABLE_WRITE_PROTECT

    // Execute until no more data in Data buffer
    while(moreDataToRead)
    {
        adr_hi = currentAddress >> 8;       // calculate high byte
        adr_lo = currentAddress & 0xFF;     // and low byte of address

        // Chop data down to 64-byte packets to be transmitted at a time
        // Maintain pointer of current startaddress
        if(currentSize > MAXPAGEWRITE)
        {
            bufferPtr = bufferPtr + MAXPAGEWRITE;
            counterI2cBuffer = MAXPAGEWRITE - 1;
            PtrTransmit = MAXPAGEWRITE + 1; // set I2CBufferArray Pointer
            currentSize = currentSize - MAXPAGEWRITE;
            currentAddress = currentAddress + MAXPAGEWRITE;

            // Get start address
            I2CBufferArray[MAXPAGEWRITE + 1] = adr_hi; // High byte address.
            I2CBufferArray[MAXPAGEWRITE] = adr_lo; // Low byte address.
        }
        else
        {
            bufferPtr = bufferPtr + currentSize;
            counterI2cBuffer = currentSize - 1;
            PtrTransmit = currentSize + EEPROM_ADDRESS_BYTE_SIZE - 1;        // set I2CBufferArray Pointer.
            moreDataToRead = 0;
            currentAddress = currentAddress + currentSize;

            // Get start address
            I2CBufferArray[currentSize + 1] = adr_hi; // High byte address.
            I2CBufferArray[currentSize] = adr_lo; // Low byte address.
        }

        // Copy data to I2CBufferArray
        uint8_t temp;
        for(i; i < bufferPtr; i++)
        {
            temp = Data[i];                 // Required or else IAR throws a
                                            // warning [Pa082]
            I2CBufferArray[counterI2cBuffer] = temp;
            counterI2cBuffer--;
        }

        I2CWriteInit();
        UCB0CTLW0 |= UCTXSTT;               // start condition generation
                                            // => I2C communication is started
        __bis_SR_register(LPM0_bits + GIE); // Enter LPM0 w/ interrupts
        UCB0CTLW0 |= UCTXSTP;               // I2C stop condition
        while(UCB0CTLW0 & UCTXSTP)
        {
            ;                               // Ensure stop condition got sent
        }
        EEPROM_AckPolling();                // Ensure data is written in EEPROM
    }

    ENABLE_WRITE_PROTECT
}

/*----------------------------------------------------------------------------*/
// Description:
//   Current Address Read Operation. Data is read from the EEPROM. The current
//   address from the EEPROM is used.
/*----------------------------------------------------------------------------*/
uint8_t EEPROM_CurrentAddressRead(void)
{
    while(UCB0STATW & UCBUSY)
    {
        ;                                   // wait until I2C module has
    }
    // finished all operations
    I2CReadInit();

    UCB0CTLW0 |= UCTXSTT;                   // I2C start condition
    while(UCB0CTLW0 & UCTXSTT)
    {
        ;                                   // Start condition sent?
    }
    UCB0CTLW0 |= UCTXSTP;                   // I2C stop condition
    __bis_SR_register(LPM0_bits + GIE);     // Enter LPM0 w/ interrupts
    while(UCB0CTLW0 & UCTXSTP)
    {
        ;                                   // Ensure stop condition got sent
    }
    return(I2CBuffer);
}

/*----------------------------------------------------------------------------*/
// Description:
//   Random Read Operation. Data is read from the EEPROM. The EEPROM
//   address is defined with the parameter Address.
/*----------------------------------------------------------------------------*/
uint8_t EEPROM_RandomRead(uint16_t Address)
{
    uint8_t adr_hi;
    uint8_t adr_lo;

    while(UCB0STATW & UCBUSY)
    {
        ;                                   // wait until I2C module has
    }
    // finished all operations

    adr_hi = Address >> 8;                  // calculate high byte
    adr_lo = Address & 0xFF;                // and low byte of address

    I2CBufferArray[1] = adr_hi;             // store single bytes that have to
    I2CBufferArray[0] = adr_lo;             // be sent in the I2CBuffer.
    PtrTransmit = EEPROM_ADDRESS_BYTE_SIZE - 1; // set I2CBufferArray Pointer

    // Write Address first
    I2CWriteInit();
    UCB0CTLW0 |= UCTXSTT;                   // start condition generation
                                            // => I2C communication is started
    __bis_SR_register(LPM0_bits + GIE);     // Enter LPM0 w/ interrupts

    // Read Data byte
    I2CReadInit();

    UCB0CTLW0 |= UCTXSTT;                   // I2C start condition
    while(UCB0CTLW0 & UCTXSTT)
    {
        ;                                   // Start condition sent?
    }
    UCB0CTLW0 |= UCTXSTP;                   // I2C stop condition
    __bis_SR_register(LPM0_bits + GIE);     // Enter LPM0 w/ interrupts
    while(UCB0CTLW0 & UCTXSTP)
    {
        ;                                   // Ensure stop condition got sent
    }
    return(I2CBuffer);
}

/*----------------------------------------------------------------------------*/
// Description:
//   Sequential Read Operation. Data is read from the EEPROM in a sequential
//   form from the parameter address as a starting point. Specify the size to
//   be read and populate to a Data buffer.
/*----------------------------------------------------------------------------*/
void EEPROM_SequentialRead(uint16_t Address,
                           uint8_t * Data,
                           uint16_t Size)
{
    uint8_t adr_hi;
    uint8_t adr_lo;
    uint16_t counterSize;

    while(UCB0STATW & UCBUSY)
    {
        ;                                   // wait until I2C module has
    }
    // finished all operations

    adr_hi = Address >> 8;                  // calculate high byte
    adr_lo = Address & 0xFF;                // and low byte of address

    I2CBufferArray[1] = adr_hi;             // store single bytes that have to
    I2CBufferArray[0] = adr_lo;             // be sent in the I2CBuffer.
    PtrTransmit = EEPROM_ADDRESS_BYTE_SIZE - 1; // set I2CBufferArray Pointer

    // Write Address first
    I2CWriteInit();
    UCB0CTLW0 |= UCTXSTT;                   // start condition generation
                                            // => I2C communication is started
    __bis_SR_register(LPM0_bits + GIE);     // Enter LPM0 w/ interrupts

    // Read Data byte
    I2CReadInit();

    UCB0CTLW0 |= UCTXSTT;                   // I2C start condition
    while(UCB0CTLW0 & UCTXSTT)
    {
        ;                                   // Start condition sent?
    }
    for(counterSize = 0; counterSize < Size; counterSize++)
    {
        __bis_SR_register(LPM0_bits + GIE); // Enter LPM0 w/ interrupts
        Data[counterSize] = I2CBuffer;
        if(counterSize == (Size - 2))
        {
            UCB0CTLW0 |= UCTXSTP;           // I2C stop condition
        }
    }
    while(UCB0CTLW0 & UCTXSTP)
    {
        ;                                   // Ensure stop condition got sent
    }
}

/*----------------------------------------------------------------------------*/
// Description:
//   Acknowledge Polling. The EEPROM will not acknowledge if a write cycle is
//   in progress. It can be used to determine when a write cycle is completed.
/*----------------------------------------------------------------------------*/
void EEPROM_AckPolling(void)
{
    while(UCB0STATW & UCBUSY)
    {
        ;                                   // wait until I2C module has
    }
    // finished all operations
    do
    {
        UCB0IFG = 0x00;                     // clear I2C interrupt flags
        UCB0CTLW0 |= UCTR;                  // I2CTRX=1 => Transmit Mode (R/W bit = 0)
        UCB0CTLW0 &= ~UCTXSTT;
        UCB0CTLW0 |= UCTXSTT;               // start condition is generated
        while(UCB0CTLW0 & UCTXSTT)          // wait till I2CSTT bit was cleared
        {
            if(!(UCB0IFG & UCNACKIFG))      // Break out if ACK received
            {
                break;
            }
        }
        UCB0CTLW0 |= UCTXSTP;               // stop condition is generated after
                                            // slave address was sent => I2C communication is started
        while(UCB0CTLW0 & UCTXSTP)
        {
            ;                               // wait till stop bit is reset
        }
        __delay_cycles(500);                // Software delay
    }
    while(UCB0IFG & UCNACKIFG);
}

/*----------------------------------------------------------------------------*/
// Description:
//   Sequential Read Operation. Data is read from the EEPROM in a sequential
//   form from the parameter address as a starting point. Specify the size to
//   be read and populate to a Data buffer.
/*----------------------------------------------------------------------------*/
void SENSOR_getData(uint8_t Address,
                    uint8_t * Data,
                    uint16_t Size)
{
    uint16_t counterSize;

    while(UCB0STATW & UCBUSY)
    {
        ;                                   // wait until I2C module has
    }
    // finished all operations

    I2CBufferArray[0] = Address;            // be sent in the I2CBuffer.
    PtrTransmit = 0;                        // set I2CBufferArray Pointer

    // Write Address first
    I2CWriteInit();
    UCB0CTLW0 |= UCTXSTT;                   // start condition generation
                                            // => I2C communication is started
    __bis_SR_register(LPM0_bits + GIE);     // Enter LPM0 w/ interrupts

    // Read Data byte
    I2CReadInit();

    UCB0CTLW0 |= UCTXSTT;                   // I2C start condition
    while(UCB0CTLW0 & UCTXSTT)
    {
        ;                                   // Start condition sent?
    }
    for(counterSize = 0; counterSize < Size; counterSize++)
    {
        __bis_SR_register(LPM0_bits + GIE); // Enter LPM0 w/ interrupts
        Data[counterSize] = I2CBuffer;
        if(counterSize == (Size - 2))
        {
            UCB0CTLW0 |= UCTXSTP;           // I2C stop condition
        }
    }
    while(UCB0CTLW0 & UCTXSTP)
    {
        ;                                   // Ensure stop condition got sent
    }
}

/*---------------------------------------------------------------------------*/
/*  Interrupt Service Routines                                               */
/*     Note that the Compiler version is checked in the following code and   */
/*     depending of the Compiler Version the correct Interrupt Service       */
/*     Routine definition is used.                                           */
#pragma vector=USCI_B0_VECTOR
__interrupt void TX_ISR_I2C(void)
{
    switch(__even_in_range(UCB0IV, USCI_I2C_UCBIT9IFG))
    {
    case USCI_NONE:          break;         // Vector 0: No interrupts
    case USCI_I2C_UCALIFG:   break;         // Vector 2: ALIFG
    case USCI_I2C_UCNACKIFG:                // Vector 4: NACKIFG
        break;
    case USCI_I2C_UCSTTIFG:  break;         // Vector 6: STTIFG
    case USCI_I2C_UCSTPIFG:  break;         // Vector 8: STPIFG
    case USCI_I2C_UCRXIFG3:  break;         // Vector 10: RXIFG3
    case USCI_I2C_UCTXIFG3:  break;         // Vector 12: TXIFG3
    case USCI_I2C_UCRXIFG2:  break;         // Vector 14: RXIFG2
    case USCI_I2C_UCTXIFG2:  break;         // Vector 16: TXIFG2
    case USCI_I2C_UCRXIFG1:  break;         // Vector 18: RXIFG1
    case USCI_I2C_UCTXIFG1:  break;         // Vector 20: TXIFG1
    case USCI_I2C_UCRXIFG0:                 // Vector 22: RXIFG0
        I2CBuffer = UCB0RXBUF;                  // store received data in buffer
        __bic_SR_register_on_exit(LPM0_bits);   // Exit LPM0
        break;
    case USCI_I2C_UCTXIFG0:                 // Vector 24: TXIFG0
        UCB0TXBUF = I2CBufferArray[PtrTransmit]; // Load TX buffer
        PtrTransmit--;                          // Decrement TX byte counter
        if(PtrTransmit < 0)
        {
            while(!(UCB0IFG & UCTXIFG0))
            {
                ;
            }
            UCB0IE &= ~UCTXIE0;             // disable interrupts.
            UCB0IFG &= ~UCTXIFG0;           // Clear USCI_B0 TX int flag
            __bic_SR_register_on_exit(LPM0_bits); // Exit LPM0
        }
        break;
    default: break;
    }
}

#endif
