/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*******************************************************************************
   Description:    Communication with an EEPROM (e.g. 24128) via I2C bus
      The I2C module of the MSP430F2274 is used to communicate with the EEPROM.
      The "Byte Write", "Page Write", "Current Address Read", "Random Read",
      "Sequential Read" and "Acknowledge Polling" commands or the EEPROM are
      realized.


*******************************************************************************/

#include <msp430.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include "I2Croutines.h"
#if (EEPROM_ADDRESS_BYTE_SIZE == 2)
#define     MAX_EEPROM_SIZE         0xFFFE
#elif (EEPROM_ADDRESS_BYTE_SIZE == 3)
//#define     MAX_EEPROM_SIZE 0x1D37F     /* 128k */
#define     MAX_EEPROM_SIZE 0x3D37F     /* 244k */
#else
#error "Please define EEPROM_ADDRESS_BYTE_SIZE=2/3 in C-preprocessor
#endif

#define     SLAVE_ADDRESS_EEPROM    0x50
#define     SLAVE_ADDRESS_SENSOR    0x51

#if defined(__IAR_SYSTEMS_ICC__)
__persistent uint8_t read_val[2048];
__persistent uint8_t write_val[2048];
__persistent uint16_t error;
__persistent uint16_t testCoverage;
__persistent uint32_t addressIndex;
__persistent uint8_t value;
__persistent uint8_t check;
#elif defined(__TI_COMPILER_VERSION__)
#pragma PERSISTENT(read_val)
uint8_t read_val[2048] = {0};
#pragma PERSISTENT(write_val)
uint8_t write_val[2048] = {0};
#pragma PERSISTENT(error)
uint16_t error = 0;
#pragma PERSISTENT(testCoverage)
uint16_t testCoverage = 0;
#pragma PERSISTENT(addressIndex)
uint32_t addressIndex = 0;
#pragma PERSISTENT(value)
uint8_t value = 0;
#pragma PERSISTENT(check)
uint8_t check = 0;
#endif

volatile float vccValue;
volatile float temperatureC;
volatile float temperatureF;

static uint16_t testEepromMemory(void);

int main(void)
{
    WDTCTL = WDTPW + WDTHOLD;               // Stop watchdog timer

    P1OUT &= ~BIT0;
    P1DIR |= BIT0;

    P4OUT &= ~BIT6;
    P4DIR |= BIT6;

    // Clock System Setup for 8MHz
    CSCTL0_H = CSKEY >> 8;                  // Unlock CS registers
    CSCTL1 = DCOFSEL_6;                     // Set DCO to 8MHz
    CSCTL2 = SELA__VLOCLK | SELS__DCOCLK | SELM__DCOCLK;
    // Set SMCLK = MCLK = DCO
    // ACLK = VLOCLK
    CSCTL3 = DIVA__1 | DIVS__1 | DIVM__1;   // Set all dividers to 1
    CSCTL0_H = 0;                           // Lock CS registers

    // Disable the GPIO power-on default high-impedance mode to activate
    // previously configured port settings
    PM5CTL0 &= ~LOCKLPM5;

    // Initialize I2C for secondary sensing functionality
    InitI2C(SLAVE_ADDRESS_SENSOR, SCL_CLOCK_DIV_100k);

    // Get current VCC
    SENSOR_getData(SENSOR_HEADER_VCC, read_val, sizeof(float));
    memcpy((void *) &vccValue, (char *) read_val, sizeof(float));

    if(3.20 > vccValue > 3.40)
    {
        // Error detected. Stop.
        P4OUT |= BIT6;
        while(1)
        {
            ;
        }
    }

    // Get current temperature in C
    SENSOR_getData(SENSOR_HEADER_TEMP_C, read_val, sizeof(float));
    memcpy((void *) &temperatureC, (char *) read_val, sizeof(float));

    // Get current temperature in F
    SENSOR_getData(SENSOR_HEADER_TEMP_F, read_val, sizeof(float));
    memcpy((void *) &temperatureF, (char *) read_val, sizeof(float));

    error = testEepromMemory();

    if(error)
    {
        printf("testCoverage = %d; Error #%d\r\n", testCoverage, error);
        P4OUT |= BIT6;
    }
    else
    {
        printf("No error found\r\n");
    }

    while(1)
    {
        P1OUT ^= BIT0;
        __delay_cycles(1000000);
    }
}

static uint16_t testEepromMemory(void)
{
    uint16_t status = 0;
    uint16_t i;
    uint8_t mode;

    // test the EEPROM portion
    for(testCoverage = 0; testCoverage < 2; testCoverage++)
    {
        if(testCoverage == 0)
        {
            mode = SCL_CLOCK_DIV_100k;
            printf("I2C Clock 100kHz\r\n");
        }
        else
        {
            mode = SCL_CLOCK_DIV_400k;
            printf("I2C Clock 400kHz\r\n");
        }

        InitI2C(SLAVE_ADDRESS_EEPROM, mode);                    // Initialize I2C module @ 100kHz

        // Write byte to EEPROM
        EEPROM_ByteWrite(0x0800, 0x01);
        // Wait for EEPROM write cycle completion
        EEPROM_AckPolling();

        // Write byte to EEPROM
        EEPROM_ByteWrite(0xD800, 0xF5);
        // Wait for EEPROM write cycle completion
        EEPROM_AckPolling();

        check = EEPROM_RandomRead(0x0800);

        // Write above the EEPROM size to see what happens
        EEPROM_ByteWrite(MAX_EEPROM_SIZE + 5, 0xAA);

        // Wait for EEPROM write cycle completion
        EEPROM_AckPolling();

        // Initializes the value to send
        value = 4;

        for(addressIndex = 0; addressIndex <= MAX_EEPROM_SIZE; addressIndex++)
        {
            // Write byte to EEPROM
            EEPROM_ByteWrite(addressIndex, value++);

            // Wait for EEPROM write cycle completion
            EEPROM_AckPolling();
            if(addressIndex == 0xd800)
            {
                __no_operation();
            }

            if(addressIndex == 0x720A)
            {
                __no_operation();
            }

            // Blink P1.0 LED every single time the operation is done to
            // indicate if the processor is still running
            P1OUT ^= BIT0;
        }

        // Re-initializes the value to check
        value = 4;

        for(addressIndex = 0; addressIndex <= MAX_EEPROM_SIZE; addressIndex++)
        {
            // Reach the value now
            check = EEPROM_RandomRead(addressIndex);

            if(check != value)
            {
                // Error detected. Stop.
                return(1);
            }

            // Increment the value
            value++;

            // Blink P1.0 LED every single time the operation is done to
            // indicate if the processor is still running
            P1OUT ^= BIT0;
        }

        check = EEPROM_RandomRead(0x0800);

        value = 0x05;

        check = EEPROM_CurrentAddressRead(); // Read from address 0x0801
        if(check != value)
        {
            return(2);
        }

        // Insert a seed value
        value = 6;

        // Fill write_val array with counter values
        for(i = 0; i < sizeof(write_val); i++)
        {
            write_val[i] = value++;
        }

        // Set starting address at 0
        addressIndex = 0x00000000;

        // Write a sequence of data array
        EEPROM_PageWrite(addressIndex, write_val, sizeof(write_val));

        // Read out a sequence of data from EEPROM
        EEPROM_SequentialRead(addressIndex, read_val, sizeof(read_val));

        // Check if the read back array is the same
        for(i = 0; i < sizeof(write_val); i++)
        {
            if(read_val[i] != write_val[i])
            {
                return(3);
            }
        }
    }
    return(status);
}
