/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*******************************************************************************
   Description:    Communication with an EEPROM (e.g. 24128) via I2C bus
      The I2C module of the MSP430F2274 is used to communicate with the EEPROM.
      The "Byte Write", "Page Write", "Current Address Read", "Random Read",
      "Sequential Read" and "Acknowledge Polling" commands or the EEPROM are
      realized.

********************************************************************************
    updates
      Jan 2005:
          - updated initialization sequence
      March 2009:
          - updated code for 2xx USCI module
          - added Page Write and Sequential Read functions
      March 2011:
          - Fixed Random and Sequential Reads to Restart
          - Added Page Write to support greater than 256 bytes
********************************************************************************

   W. Goh
   Texas Instruments Inc.
   March 2011
   IAR Embedded Workbench Version: 5.20.1

********************************************************************************

   Copyright 2007-2009 Texas Instruments Incorporated. All rights reserved.

   IMPORTANT: Your use of this Software is limited to those specific rights
   granted under the terms of a software license agreement between the user who
   downloaded the software, his/her employer (which must be your employer) and
   Texas Instruments Incorporated (the "License"). You may not use this Software
   unless you agree to abide by the terms of the License. The License limits your
   use, and you acknowledge, that the Software may not be modified, copied or
   distributed unless embedded on a Texas Instruments microcontroller or used
   solely and exclusively in conjunction with a Texas Instruments radio frequency
   transceiver, which is integrated into your product. Other than for the
   foregoing purpose, you may not use, reproduce, copy, prepare derivative works
   of, modify, distribute, perform, display or sell this Software and/or its
   documentation for any purpose.

   YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
   PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
   INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
   NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL TEXAS
   INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
   NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL
   EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT
   LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR CONSEQUENTIAL
   DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS,
   TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
   LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

   Should you have any questions regarding your right to use this Software,
   contact Texas Instruments Incorporated at www.TI.com.
*******************************************************************************/

#include <msp430.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include "spi_routines.h"
#if (EEPROM_ADDRESS_BYTE_SIZE == 2)
#define     MAX_EEPROM_SIZE         0x237F      /* ~8k */
#elif (EEPROM_ADDRESS_BYTE_SIZE == 3)
//#define     MAX_EEPROM_SIZE         0x1D37F     /* 128k */
#define     MAX_EEPROM_SIZE         0x3D37F     /* 256k */
#else
#error "Please define EEPROM_ADDRESS_BYTE_SIZE=2/3 in C-preprocessor
#endif

#define     SLAVE_ADDRESS_EEPROM    0x50
#define     SLAVE_ADDRESS_SENSOR    0x51

#if defined(__IAR_SYSTEMS_ICC__)
__persistent uint8_t read_val[2048];
__persistent uint8_t write_val[2048];
__persistent uint16_t error;
__persistent uint16_t testCoverage;
__persistent uint32_t addressIndex;
__persistent uint8_t value;
__persistent uint8_t check;
#elif defined(__TI_COMPILER_VERSION__)
#pragma PERSISTENT(read_val)
uint8_t read_val[2048] = {0};
#pragma PERSISTENT(write_val)
uint8_t write_val[2048] = {0};
#pragma PERSISTENT(error)
uint16_t error = 0;
#pragma PERSISTENT(testCoverage)
uint16_t testCoverage = 0;
#pragma PERSISTENT(addressIndex)
uint32_t addressIndex = 0;
#pragma PERSISTENT(value)
uint8_t value = 0;
#pragma PERSISTENT(check)
uint8_t check = 0;
#endif

volatile float vccValue;
volatile float temperatureC;
volatile float temperatureF;

static uint16_t testEepromMemory(void);

int main(void)
{
    WDTCTL = WDTPW + WDTHOLD;               // Stop watchdog timer

    P1OUT &= ~(BIT0 | BIT2);
    P1DIR |= BIT0 | BIT2;

    P4OUT &= ~BIT6;
    P4DIR |= BIT6;

    // Clock System Setup for 8MHz
    CSCTL0_H = CSKEY >> 8;                  // Unlock CS registers
    CSCTL1 = DCOFSEL_6;                     // Set DCO to 8MHz
    CSCTL2 = SELA__VLOCLK | SELS__DCOCLK | SELM__DCOCLK;
    // Set SMCLK = MCLK = DCO
    // ACLK = VLOCLK
    CSCTL3 = DIVA__1 | DIVS__1 | DIVM__1;   // Set all dividers to 1
    CSCTL0_H = 0;                           // Lock CS registers

    // Disable the GPIO power-on default high-impedance mode to activate
    // previously configured port settings
    PM5CTL0 &= ~LOCKLPM5;

    // Initialize SPI for secondary sensing functionality
#if (EEPROM_ADDRESS_BYTE_SIZE == 3)
    SPI_Init(SPI_SCLK_CLOCK_1M);
#else
    SPI_Init(SPI_SCLK_CLOCK_300k);
#endif

    // Get current VCC
    SENSOR_getData(SENSOR_HEADER_VCC, read_val, sizeof(float));
    memcpy((void *) &vccValue, (char *) read_val, sizeof(float));

    if(3.20 > vccValue > 3.40)
    {
        // Error detected. Stop.
        P4OUT |= BIT6;
        while(1)
        {
            ;
        }
    }

    // Get current temperature in C
    SENSOR_getData(SENSOR_HEADER_TEMP_C, read_val, sizeof(float));
    memcpy((void *) &temperatureC, (char *) read_val, sizeof(float));

    // Get current temperature in F
    SENSOR_getData(SENSOR_HEADER_TEMP_F, read_val, sizeof(float));
    memcpy((void *) &temperatureF, (char *) read_val, sizeof(float));

    error = testEepromMemory();

    if(error)
    {
        printf("testCoverage = %d; Error #%d\r\n", testCoverage, error);
        P4OUT |= BIT6;
    }
    else
    {
        printf("No error found\r\n");
    }

    while(1)
    {
        P1OUT ^= BIT0;
        __delay_cycles(1000000);
    }
}

static uint16_t testEepromMemory(void)
{
    uint16_t status = 0;
    uint16_t i;
    uint32_t mode;

    // test the EEPROM portion
    for(testCoverage = 0; testCoverage < 3; testCoverage++)
    {
        printf("Running Test at ");
        switch(testCoverage)
        {
#if (EEPROM_ADDRESS_BYTE_SIZE == 3)
        case 0:
            mode = SPI_SCLK_CLOCK_1M;
            printf("SPI Clock 1MHz\r\n");
            break;
        case 1:
            mode = SPI_SCLK_CLOCK_500k;
            printf("SPI Clock 500kHz\r\n");
            break;
        case 2:
            mode = SPI_SCLK_CLOCK_250k;
            printf("SPI Clock 250kHz\r\n");
            break;
#else
        case 0:
        case 1:
        case 2:
            mode = SPI_SCLK_CLOCK_250k;
            printf("SPI Clock 250kHz\r\n");
            break;
#endif
        }

        // Initialize SPI module with a desired speed rate to test
        SPI_Init(mode);

        // Write byte to EEPROM
        EEPROM_ByteWrite(0x000800, 0x01);
        // Wait for EEPROM write cycle completion
        EEPROM_AckPolling();

        // Reach the value now
        check = EEPROM_RandomRead(0x000800);

        if(check != 0x01)
        {
            return(1);
        }

#if (EEPROM_ADDRESS_BYTE_SIZE == 3)
        // Write byte to EEPROM
        EEPROM_ByteWrite(0xD800, 0xF5);
        // Wait for EEPROM write cycle completion
        EEPROM_AckPolling();

        check = EEPROM_RandomRead(0xD800);

        if(check != 0xF5)
        {
            return(2);
        }

        // Insert a seed value
        value = 6;

        // Fill write_val array with counter values
        for(i = 0; i < 10; i++)
        {
            write_val[i] = value++;
        }

        // Set starting address at 0
        addressIndex = 0x00000000;

        // Write a sequence of data array
        EEPROM_PageWrite(addressIndex, write_val, 10);

        // Read out a sequence of data from EEPROM
        EEPROM_SequentialRead(addressIndex, read_val, 10);

        // Check if the read back array is the same
        for(i = 0; i < 10; i++)
        {
            if(read_val[i] != write_val[i])
            {
                return(3);
            }
        }

        // Test memory write at the boundary location of where EEPROM1 size ends
        // and EEPROM2 size begins
        // Page write at 0x937F
        addressIndex = 0x0000937F;

        // Write a sequence of data array
        EEPROM_PageWrite(addressIndex, write_val, 10);

        // Read out a sequence of data from EEPROM
        EEPROM_SequentialRead(addressIndex, read_val, 10);

        // Check if the read back array is the same
        for(i = 0; i < 10; i++)
        {
            if(read_val[i] != write_val[i])
            {
                return(4);
            }
        }

        // Page write almost at the end of eeprom memory and overflows the
        // upper memory index test
        // Page write starting at 0x0003D37D. And it should only write to
        // 0x0003D37F.
        addressIndex = 0x0003D37D;

        // Write a sequence of data array
        EEPROM_PageWrite(addressIndex, write_val, 10);

        // Read out a sequence of data from EEPROM
        EEPROM_SequentialRead(addressIndex, read_val, 3);

        // Check if the read back array is the same
        for(i = 0; i < 3; i++)
        {
            if(read_val[i] != write_val[i])
            {
                return(5);
            }
        }

#endif

        // Write above the EEPROM size to see what happens
        EEPROM_ByteWrite(MAX_EEPROM_SIZE + 5, 0xAA);

        // Initializes the value to send
        value = 4;

        for(addressIndex = 0; addressIndex <= MAX_EEPROM_SIZE; addressIndex++)
        {
            // Write byte to EEPROM
            EEPROM_ByteWrite(addressIndex, value++);

            if(addressIndex == 0x214F0)
            {
                __no_operation();
            }
            // Blink P1.0 LED every single time the operation is done to
            // indicate if the processor is still running
            P1OUT ^= BIT0;
        }

        // Re-initializes the value to check
        value = 4;

        for(addressIndex = 0; addressIndex <= MAX_EEPROM_SIZE; addressIndex++)
        {
            // Reach the value now
            check = EEPROM_RandomRead(addressIndex);

            if(check != value)
            {
                // Error detected. Stop.
                return(6);
            }

            // Increment the value
            value++;

            // Blink P1.0 LED every single time the operation is done to
            // indicate if the processor is still running
            P1OUT ^= BIT0;
        }

        check = EEPROM_RandomRead(0x0800);

        // Insert a seed value
        value = 6;

        // Fill write_val array with counter values
        for(i = 0; i < sizeof(write_val); i++)
        {
            write_val[i] = value++;
        }

        // Set starting address at 0
        addressIndex = 0x00000000;

        // Write a sequence of data array
        EEPROM_PageWrite(addressIndex, write_val, sizeof(write_val));

        // Read out a sequence of data from EEPROM
        EEPROM_SequentialRead(addressIndex, read_val, sizeof(read_val));

        // Check if the read back array is the same
        for(i = 0; i < sizeof(write_val); i++)
        {
            if(read_val[i] != write_val[i])
            {
                return(7);
            }
        }
    }

    return(status);
}
