/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/******************************************************************************/
/*  Communication with an EEPROM (e.g. 2465) via I2C bus                      */
/*  The I2C module of the MSP430F2274 is used to communicate with the EEPROM. */
/*  The "Byte Write", "Page Write", "Current Address Read", "Random Read",    */
/*  "Sequential Read" and "Acknowledge Polling" commands or the EEPROM are    */
/*  realized.                                                                 */
/*                                                                            */
/*  developed with IAR Embedded Workbench V5.20.1                             */
/*                                                                            */
/*  Texas Instruments                                                         */
/*  William Goh                                                               */
/*  March 2011                                                                */
/*----------------------------------------------------------------------------*/
/*  updates                                                                   */
/*    Jan 2005:                                                               */
/*        - updated initialization sequence                                   */
/*    March 2009:                                                             */
/*        - updated code for 2xx USCI module                                  */
/*        - added Page Write and Sequential Read functions                    */
/*    March 2011:                                                             */
/*        - Fixed Random and Sequential Reads to Restart                      */
/*        - Added Page Write to support greater than 256 bytes                */
/*******************************************************************************
   ;
   ; THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR
   ; REPRESENTATIONS, EITHER EXPRESS, IMPLIED OR STATUTORY,
   ; INCLUDING ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
   ; FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY OR
   ; COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE.
   ; TI DISCLAIMS ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET
   ; POSSESSION, AND NON-INFRINGEMENT OF ANY THIRD PARTY
   ; INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR
   ; YOUR USE OF THE PROGRAM.
   ;
   ; IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
   ; CONSEQUENTIAL OR INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY
   ; THEORY OF LIABILITY AND WHETHER OR NOT TI HAS BEEN ADVISED
   ; OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY OUT
   ; OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM.
   ; EXCLUDED DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF
   ; REMOVAL OR REINSTALLATION, COMPUTER TIME, LABOR COSTS, LOSS
   ; OF GOODWILL, LOSS OF PROFITS, LOSS OF SAVINGS, OR LOSS OF
   ; USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S
   ; AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF
   ; YOUR USE OF THE PROGRAM EXCEED FIVE HUNDRED DOLLARS
   ; (U.S.$500).
   ;
   ; Unless otherwise stated, the Program written and copyrighted
   ; by Texas Instruments is distributed as "freeware".  You may,
   ; only under TI's copyright in the Program, use and modify the
   ; Program without any charge or restriction.  You may
   ; distribute to third parties, provided that you transfer a
   ; copy of this license to the third party and the third party
   ; agrees to these terms by its first use of the Program. You
   ; must reproduce the copyright notice and any other legend of
   ; ownership on each copy or partial copy, of the Program.
   ;
   ; You acknowledge and agree that the Program contains
   ; copyrighted material, trade secrets and other TI proprietary
   ; information and is protected by copyright laws,
   ; international copyright treaties, and trade secret laws, as
   ; well as other intellectual property laws.  To protect TI's
   ; rights in the Program, you agree not to decompile, reverse
   ; engineer, disassemble or otherwise translate any object code
   ; versions of the Program to a human-readable form.  You agree
   ; that in no event will you alter, remove or destroy any
   ; copyright notice included in the Program.  TI reserves all
   ; rights not specifically granted under this license. Except
   ; as specifically provided herein, nothing in this agreement
   ; shall be construed as conferring by implication, estoppel,
   ; or otherwise, upon you, any license or other right under any
   ; TI patents, copyrights or trade secrets.
   ;
   ; You may not use the Program in non-TI devices.
   ;
 ******************************************************************************/
#include <msp430.h>
#include <driverlib.h>
#include <stdint.h>
#include "spi_routines.h"

// This number can be as large as the target EEPROM size. But we're limited by
// the memory availabilty to create an array that is large.
#define  MAXPAGEWRITE   10

#if (EEPROM_ADDRESS_BYTE_SIZE == 2)
#define CS_DELAY                    100
#else
#define CS_DELAY                    108
#endif
#define SPI_ADD_DATA_DELAY_1M       115
#define SPI_ADD_DATA_DELAY_500k     95
#define SPI_ADD_DATA_DELAY_250k     130

int PtrTransmit;
uint8_t SPIBufferArray[MAXPAGEWRITE + EEPROM_ADDRESS_BYTE_SIZE + 2];
uint8_t spiBuffer;
extern uint16_t testCoverage;

/*----------------------------------------------------------------------------*/
// Description:
//   Initialization of the SPI Module
/*----------------------------------------------------------------------------*/
void SPI_Init(uint32_t eeprom_spi_speed)
{
    EUSCI_B_SPI_initMasterParam param = {
        EUSCI_B_SPI_CLOCKSOURCE_SMCLK,
        __SYSTEM_CLOCK__,
        eeprom_spi_speed,
        EUSCI_B_SPI_MSB_FIRST,
        EUSCI_B_SPI_PHASE_DATA_CHANGED_ONFIRST_CAPTURED_ON_NEXT,
        EUSCI_B_SPI_CLOCKPOLARITY_INACTIVITY_HIGH,
        EUSCI_B_SPI_3PIN
    };

    // Configure UCB0SIMO and UCB0SOMI on P1.6 and P1.7
    GPIO_setAsPeripheralModuleFunctionInputPin(
        SPI_MOSI_MISO_PORT,
        SPI_MOSI_MISO_PIN,
        GPIO_SECONDARY_MODULE_FUNCTION);

    // Configure UCB0CLK on P2.2
    GPIO_setAsPeripheralModuleFunctionInputPin(
        SPI_SCLK_PORT,
        SPI_SCLK_PIN,
        GPIO_SECONDARY_MODULE_FUNCTION);

    // Use this bit for write-protect enable/disable
    GPIO_setOutputHighOnPin(WP_PORT, WP_PIN);
    GPIO_setAsOutputPin(WP_PORT, WP_PIN);

    // SPI CSn pin. High by default.
    GPIO_setOutputHighOnPin(SPI_CSn_PORT, SPI_CSn_PIN);
    GPIO_setAsOutputPin(SPI_CSn_PORT, SPI_CSn_PIN);

    // eUSCI SPI configuration for Master mode
    EUSCI_B_SPI_initMaster(EEPROM_SPI_EUSCI_BASE,
                           (EUSCI_B_SPI_initMasterParam *) &param);
    EUSCI_B_SPI_enable(EEPROM_SPI_EUSCI_BASE);
}

/*----------------------------------------------------------------------------*/
// Description:
//   Byte Write Operation. The communication via the I2C bus with an EEPROM
//   (2465) is realized. A data byte is written into a user defined address.
/*----------------------------------------------------------------------------*/
#if (EEPROM_ADDRESS_BYTE_SIZE == 2)
void EEPROM_ByteWrite(uint16_t Address,
                      uint8_t Data)
#elif (EEPROM_ADDRESS_BYTE_SIZE == 3)
void EEPROM_ByteWrite(uint32_t Address,
                      uint8_t Data)
#endif
{
    EEPROM_PageWrite(Address, &Data, 1);
}

/*----------------------------------------------------------------------------*/
// Description:
//   Page Write Operation. The communication via the I2C bus with an EEPROM
//   (24xx65) is realized. A data byte is written into a user defined address.
/*----------------------------------------------------------------------------*/
#if (EEPROM_ADDRESS_BYTE_SIZE == 2)
void EEPROM_PageWrite(uint16_t Address,
                      uint8_t * Data,
                      uint16_t Size)
#elif (EEPROM_ADDRESS_BYTE_SIZE == 3)
void EEPROM_PageWrite(uint32_t Address,
                      uint8_t * Data,
                      uint16_t Size)
#endif
{
    volatile uint16_t i = 0;
    volatile uint16_t counterI2cBuffer;
    uint16_t currentSize = Size;

    // Preconfigure the DMA0
    // Configure the DMA trigger source
    DMACTL0 = DMA0TSEL__UCB0TXIFG0;

    // Configure DMA channel 0 to transmit
    __data16_write_addr((unsigned short) &DMA0SA,(unsigned long) Data);
    // Source block address
    __data16_write_addr((unsigned short) &DMA0DA,(unsigned long) &UCB0TXBUF);
    // Destination single address
    DMA0SZ = currentSize;                   // Block size
    DMA0CTL = DMADT_1 | DMASRCINCR_3 | DMADSTINCR_0 | DMADSTBYTE | DMASRCBYTE |
              DMALEVEL;                                                                   // Rpt, inc

    // wait until SPI module has finished all operations.
    while(EUSCI_B_SPI_isBusy(EEPROM_SPI_EUSCI_BASE))
    {
        ;
    }

    DISABLE_WRITE_PROTECT

    // Set CS = Low to start
    GPIO_setOutputLowOnPin(SPI_CSn_PORT,
                           SPI_CSn_PIN);

    __delay_cycles(CS_DELAY);

    // Send start address first
#if (EEPROM_ADDRESS_BYTE_SIZE == 2)
    SPIBufferArray[0] = (uint8_t) (Address & 0xFF); // Low byte address.
    SPIBufferArray[1] = (uint8_t) (Address >> 8);  // High byte address.
    SPIBufferArray[2] = EEPROM_WRITE_OPCODE; // Specify the op_code
    PtrTransmit = 2;
#elif (EEPROM_ADDRESS_BYTE_SIZE == 3)
    SPIBufferArray[0] = (uint8_t) (Address & 0xFF); // Low byte address.
    SPIBufferArray[1] = (uint8_t) (Address >> 8);  // High byte address.
    SPIBufferArray[2] = (uint8_t) (Address >> 16); // High byte lo nibble address.
    SPIBufferArray[3] = EEPROM_WRITE_OPCODE; // Specify the op_code
    PtrTransmit = 3;
#endif

    UCB0IFG |= UCTXIFG;

    // Make enable interrupt an atomic function
    __disable_interrupt();

    // Start the SPI transmit by enabling the interrupt flag
    EUSCI_B_SPI_enableInterrupt(EEPROM_SPI_EUSCI_BASE,
                                EUSCI_B_SPI_TRANSMIT_INTERRUPT);

    __bis_SR_register(LPM0_bits + GIE); // Enter LPM0 w/ interrupts

    // Wait until the SPI is no longer busy before disabling CS
    while(EUSCI_B_SPI_isBusy(EEPROM_SPI_EUSCI_BASE))
    {
        ;
    }

    // Provide a short delay for slave to put data in buffer before transmitting
    // dummy data bytes
    switch(testCoverage)
    {
#if (EEPROM_ADDRESS_BYTE_SIZE == 3)
    case 0:
        __delay_cycles(SPI_ADD_DATA_DELAY_1M + 15);
        break;
    case 1:
        __delay_cycles(SPI_ADD_DATA_DELAY_500k);
        break;
    case 2:
        __delay_cycles(SPI_ADD_DATA_DELAY_250k);
        break;
#else
    case 0:
    case 1:
    case 2:
        __delay_cycles(SPI_ADD_DATA_DELAY_250k);
        break;
#endif
    }

    // Enable receive DMA0
    DMA0CTL |= DMAEN | DMAREQ;

    // Wait till all data is sent
    while(!(DMAIFG & DMA0CTL))
    {
        ;
    }

    // Wait until the SPI is no longer busy before disabling CS
    while(EUSCI_B_SPI_isBusy(EEPROM_SPI_EUSCI_BASE))
    {
        ;
    }

    // Set CS = High to disable SPI transmission
    GPIO_setOutputHighOnPin(SPI_CSn_PORT, SPI_CSn_PIN);

    ENABLE_WRITE_PROTECT
}

/*----------------------------------------------------------------------------*/
// Description:
//   Current Address Read Operation is not supported.
/*----------------------------------------------------------------------------*/
uint8_t EEPROM_CurrentAddressRead(void)
{
    return(1);
}

/*----------------------------------------------------------------------------*/
// Description:
//   Random Read Operation. Data is read from the EEPROM. The EEPROM
//   address is defined with the parameter Address.
/*----------------------------------------------------------------------------*/
#if (EEPROM_ADDRESS_BYTE_SIZE == 2)
uint8_t EEPROM_RandomRead(uint16_t Address)
#elif (EEPROM_ADDRESS_BYTE_SIZE == 3)
uint8_t EEPROM_RandomRead(uint32_t Address)
#endif
{
    uint8_t data[1];

    EEPROM_SequentialRead(Address, data, 1);

    return(data[0]);
}

/*----------------------------------------------------------------------------*/
// Description:
//   Sequential Read Operation. Data is read from the EEPROM in a sequential
//   form from the parameter address as a starting point. Specify the size to
//   be read and populate to a Data buffer.
/*----------------------------------------------------------------------------*/
#if (EEPROM_ADDRESS_BYTE_SIZE == 2)
void EEPROM_SequentialRead(uint16_t Address,
                           uint8_t * Data,
                           uint16_t Size)
#elif (EEPROM_ADDRESS_BYTE_SIZE == 3)
void EEPROM_SequentialRead(uint32_t Address,
                           uint8_t * Data,
                           uint16_t Size)
#endif
{
    // Preconfigure the DMA0 and DMA1
    // Configure the DMA trigger source
    DMACTL0 = DMA0TSEL__UCB0TXIFG0 | DMA1TSEL__UCB0RXIFG0;

    // Configure DMA channel 0 to transmit
    __data16_write_addr((unsigned short) &DMA0SA,
                        (unsigned long) &SPIBufferArray[0]);
    // Source block address
    __data16_write_addr((unsigned short) &DMA0DA,(unsigned long) &UCB0TXBUF);
    // Destination single address
    DMA0SZ = Size;                              // Block size
    DMA0CTL = DMADT_1 | DMASRCINCR_0 | DMADSTINCR_0 | DMADSTBYTE | DMASRCBYTE |
              DMALEVEL;                                                                   // Rpt, inc

    // Configure DMA channel 1 to receive
    __data16_write_addr((unsigned short) &DMA1SA,(unsigned long) &UCB0RXBUF);
    // Source block address
    __data16_write_addr((unsigned short) &DMA1DA,(unsigned long) Data);
    // Destination single address
    DMA1SZ = Size;                          // Block size
    DMA1CTL = DMADT_5 | DMASRCINCR_0 | DMADSTINCR_3 | DMADSTBYTE | DMASRCBYTE |
              DMALEVEL;                                                                   // Rpt, inc

    // wait until SPI module has finished all operations.
    while(EUSCI_B_SPI_isBusy(EEPROM_SPI_EUSCI_BASE))
    {
        ;
    }

    // Set CS = Low to start
    GPIO_setOutputLowOnPin(SPI_CSn_PORT, SPI_CSn_PIN);

    __delay_cycles(CS_DELAY);

    // Send the Read op code
#if (EEPROM_ADDRESS_BYTE_SIZE == 2)
    SPIBufferArray[0] = (uint8_t) (Address & 0xFF); // Low byte address.
    SPIBufferArray[1] = (uint8_t) (Address >> 8);  // High byte address.
    SPIBufferArray[2] = EEPROM_READ_OPCODE; // Specify the op_code
    PtrTransmit = 2;
#elif (EEPROM_ADDRESS_BYTE_SIZE == 3)
    SPIBufferArray[0] = (uint8_t) (Address & 0xFF); // Low byte address.
    SPIBufferArray[1] = (uint8_t) (Address >> 8);  // High byte address.
    SPIBufferArray[2] = (uint8_t) (Address >> 16); // High byte lo nibble address.
    SPIBufferArray[3] = EEPROM_READ_OPCODE; // Specify the op_code
    PtrTransmit = 3;
#endif

    // Write Opcode & Address first
    // Start the SPI transmit by enabling the interrupt flag
    UCB0IFG |= UCTXIFG;

    // Make enable interrupt an atomic function
    __disable_interrupt();

    EUSCI_B_SPI_enableInterrupt(EEPROM_SPI_EUSCI_BASE,
                                EUSCI_B_SPI_TRANSMIT_INTERRUPT);

    __bis_SR_register(LPM0_bits + GIE); // Enter LPM0 w/ interrupts

    // Specify what dummy byte to transmit out via SPI
    SPIBufferArray[0] = 0xFF;

    // Wait until the SPI is no longer busy before disabling CS
    while(EUSCI_B_SPI_isBusy(EEPROM_SPI_EUSCI_BASE))
    {
        ;
    }

    // Provide a short delay for slave to put data in buffer before transmitting
    // dummy data bytes
    switch(testCoverage)
    {
#if (EEPROM_ADDRESS_BYTE_SIZE == 3)
    case 0:
        __delay_cycles(SPI_ADD_DATA_DELAY_1M);
        break;
    case 1:
        __delay_cycles(SPI_ADD_DATA_DELAY_500k);
        break;
    case 2:
        __delay_cycles(SPI_ADD_DATA_DELAY_250k);
        break;
#else
    case 0:
    case 1:
    case 2:
        __delay_cycles(SPI_ADD_DATA_DELAY_250k);
        break;
#endif
    }

    // Start transmitting dummy bytes to receive data

    // Clear the receive interrupt flag first
    UCB0IFG &= ~UCRXIFG;

    DMA1CTL |= DMAEN | DMAREQ;                         // Enable receive DMA0
    DMA0CTL |= DMAEN | DMAREQ;                         // Enable transmit DMA0

    while(!(DMAIFG & DMA1CTL))
    {
        ;
    }

    // Wait until the SPI is no longer busy before disabling CS
    while(EUSCI_B_SPI_isBusy(EEPROM_SPI_EUSCI_BASE))
    {
        ;
    }

    // Set CS = High to disable SPI transmission
    GPIO_setOutputHighOnPin(SPI_CSn_PORT, SPI_CSn_PIN);
}

/*----------------------------------------------------------------------------*/
// Description:
//   Acknowledge Polling is not supported by SPI routine.
//   in progress. It can be used to determine when a write cycle is completed.
/*----------------------------------------------------------------------------*/
void EEPROM_AckPolling(void)
{
    return;
}

/*----------------------------------------------------------------------------*/
// Description:
//   Sequential Read Operation. Data is read from the EEPROM in a sequential
//   form from the parameter address as a starting point. Specify the size to
//   be read and populate to a Data buffer.
/*----------------------------------------------------------------------------*/
void SENSOR_getData(uint8_t Address,
                    uint8_t * Data,
                    uint16_t Size)
{
    // Preconfigure the DMA0 and DMA1
    // Configure the DMA trigger source
    DMACTL0 = DMA0TSEL__UCB0TXIFG0 | DMA1TSEL__UCB0RXIFG0;

    // Configure DMA channel 0 to transmit
    __data16_write_addr((unsigned short) &DMA0SA,
                        (unsigned long) &SPIBufferArray[0]);
    // Source block address
    __data16_write_addr((unsigned short) &DMA0DA,(unsigned long) &UCB0TXBUF);
    // Destination single address
    DMA0SZ = Size;                              // Block size
    DMA0CTL = DMADT_1 | DMASRCINCR_0 | DMADSTINCR_0 | DMADSTBYTE | DMASRCBYTE |
              DMALEVEL;                                                                   // Rpt, inc

    // Configure DMA channel 1 to receive
    __data16_write_addr((unsigned short) &DMA1SA,(unsigned long) &UCB0RXBUF);
    // Source block address
    __data16_write_addr((unsigned short) &DMA1DA,(unsigned long) Data);
    // Destination single address
    DMA1SZ = Size;                          // Block size
    DMA1CTL = DMADT_5 | DMASRCINCR_0 | DMADSTINCR_3 | DMADSTBYTE | DMASRCBYTE |
              DMALEVEL;                                                                   // Rpt, inc

    // wait until SPI module has finished all operations.
    while(EUSCI_B_SPI_isBusy(EEPROM_SPI_EUSCI_BASE))
    {
        ;
    }

    // Set CS = Low to start
    GPIO_setOutputLowOnPin(SPI_CSn_PORT, SPI_CSn_PIN);

    __delay_cycles(CS_DELAY);

    // Send the Read op code
    SPIBufferArray[0] = Address;
    SPIBufferArray[1] = SENSOR_READ_OPCODE;
    PtrTransmit = 1;

    // Write Opcode & Address first
    // Start the SPI transmit by enabling the interrupt flag
    UCB0IFG |= UCTXIFG;

    // Make enable interrupt an atomic function
    __disable_interrupt();

    EUSCI_B_SPI_enableInterrupt(EEPROM_SPI_EUSCI_BASE,
                                EUSCI_B_SPI_TRANSMIT_INTERRUPT);

    __bis_SR_register(LPM0_bits + GIE); // Enter LPM0 w/ interrupts

    // Specify what dummy byte to transmit out via SPI
    SPIBufferArray[0] = 0xFF;

    // Wait until the SPI is no longer busy before disabling CS
    while(EUSCI_B_SPI_isBusy(EEPROM_SPI_EUSCI_BASE))
    {
        ;
    }

    // Provide a short delay for slave to put data in buffer before transmitting
    // dummy data bytes
    switch(testCoverage)
    {
#if (EEPROM_ADDRESS_BYTE_SIZE == 3)
    case 0:
        __delay_cycles(SPI_ADD_DATA_DELAY_1M);
        break;
    case 1:
        __delay_cycles(SPI_ADD_DATA_DELAY_500k);
        break;
    case 2:
        __delay_cycles(SPI_ADD_DATA_DELAY_250k);
        break;
#else
    case 0:
    case 1:
    case 2:
        __delay_cycles(SPI_ADD_DATA_DELAY_250k);
        break;
#endif
    }

    // Start transmitting dummy bytes to receive data

    // Clear the receive interrupt flag first
    UCB0IFG &= ~UCRXIFG;

    DMA1CTL |= DMAEN | DMAREQ;                         // Enable receive DMA0
    DMA0CTL |= DMAEN | DMAREQ;                         // Enable transmit DMA0

    while(!(DMAIFG & DMA1CTL))
    {
        ;
    }

    // Wait until the SPI is no longer busy before disabling CS
    while(EUSCI_B_SPI_isBusy(EEPROM_SPI_EUSCI_BASE))
    {
        ;
    }

    // Set CS = High to disable SPI transmission
    GPIO_setOutputHighOnPin(SPI_CSn_PORT, SPI_CSn_PIN);
}

/*---------------------------------------------------------------------------*/
/*  Interrupt Service Routines                                               */
/*     Note that the Compiler version is checked in the following code and   */
/*     depending of the Compiler Version the correct Interrupt Service       */
/*     Routine definition is used.                                           */
#pragma vector=USCI_B0_VECTOR
__interrupt void TX_ISR_I2C(void)
{
    switch(__even_in_range(UCB0IV, USCI_SPI_UCTXIFG))
    {
    case USCI_NONE: break;                  // Vector 0: No interrupts
    case USCI_SPI_UCRXIFG:
        // store received data in buffer
        spiBuffer = UCB0RXBUF;

        // Exit LPM0
        __bic_SR_register_on_exit(LPM0_bits);
        break;
    case USCI_SPI_UCTXIFG:
        // Load TX buffer and decrement
        EUSCI_B_SPI_transmitData(EEPROM_SPI_EUSCI_BASE,
                                 SPIBufferArray[PtrTransmit--]);

        if(PtrTransmit < 0)
        {
            while(!EUSCI_B_SPI_getInterruptStatus(EEPROM_SPI_EUSCI_BASE,
                                                  EUSCI_B_SPI_TRANSMIT_INTERRUPT))
            {
                ;
            }

            // disable interrupts.
            EUSCI_B_SPI_disableInterrupt(EEPROM_SPI_EUSCI_BASE,
                                         EUSCI_B_SPI_TRANSMIT_INTERRUPT);

            // Exit LPM0
            __bic_SR_register_on_exit(LPM0_bits);
        }
        break;
    default: break;
    }
}

//#endif
