/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#include <msp430.h>
#include <stdint.h>
#include <driverlib.h>
#include "eeprom.h"
#include "eeprom_definitions.h"

#if defined(__IAR_SYSTEMS_ICC__)
#pragma location=EEPROM1_BEGIN
__no_init uint32_t eeprom1_array[EEPROM1_LONG_SIZE];
#ifdef EEPROM2_SIZE
#pragma location=EEPROM2_BEGIN
__no_init uint32_t eeprom2_array[EEPROM2_LONG_SIZE];
#endif

// Pointers to eeprom start addresses on the lower and upper
// memory bound
__persistent void * eeprom1_start_address = eeprom1_array;
#ifdef EEPROM2_SIZE
__persistent void * eeprom2_start_address = eeprom2_array;
#endif

#ifdef EEPROM2_SIZE
__persistent uint32_t eeprom_total_size = sizeof(eeprom1_array) +
                                          sizeof(eeprom2_array);
#else
__persistent uint32_t eeprom_total_size = sizeof(eeprom1_array);
#endif

// Pointer to the current index
__persistent volatile uint32_t eepromReadIndexPointer = 0;
__persistent volatile uint32_t eepromWriteIndexPointer = 0;
__persistent uint8_t * eepromReadDataPointerAddress = 0;
__persistent uint8_t * eepromWriteDataPointerAddress = 0;

// Write-Protect state
// writeProtected = 0 - write enabled
// writeProtected = 1 - write protected
__persistent uint8_t writeProtected = 0;
#elif defined(__TI_COMPILER_VERSION__)
#pragma location=EEPROM1_BEGIN
#pragma PERSISTENT(eeprom1_array)
uint32_t eeprom1_array[EEPROM1_LONG_SIZE] = {0};
#ifdef EEPROM2_SIZE
#pragma location=EEPROM2_BEGIN
#pragma PERSISTENT(eeprom2_array)
uint32_t eeprom2_array[EEPROM2_LONG_SIZE] = {0};
#endif
// Pointers to eeprom start and end addresses on the lower and upper
// memory bound
#pragma PERSISTENT(eeprom1_start_address)
void * eeprom1_start_address = eeprom1_array;
#pragma PERSISTENT(eeprom2_start_address)
void * eeprom2_start_address = eeprom2_array;

#pragma PERSISTENT(eeprom_total_size)
uint32_t eeprom_total_size = sizeof(eeprom1_array) + sizeof(eeprom2_array);

// Pointer to the current index
#pragma PERSISTENT(eepromReadIndexPointer)
uint32_t eepromReadIndexPointer = 0;
#pragma PERSISTENT(eepromWriteIndexPointer)
uint32_t eepromWriteIndexPointer = 0;
#pragma PERSISTENT(eepromReadDataPointerAddress)
uint8_t * eepromReadDataPointerAddress = 0;
#pragma PERSISTENT(eepromWriteDataPointerAddress)
uint8_t * eepromWriteDataPointerAddress = 0;

// Write-Protect state
// writeProtected = 0 - write enabled
// writeProtected = 1 - write protected
#pragma PERSISTENT(writeProtected)
uint8_t writeProtected = 0;
#endif

uint16_t dmaSize;

static void eeprom_initializeMemoryBound(void);

void eeprom_initializeInterface(void)
{
    // Initializes the EEPROM read data pointer is within the memory region
    eeprom_initializeMemoryBound();

    // Initialize the write-protect (WP) pin
    // WP = 0 - write enabled
    // WP = 1 - write protected

    // WP pin is configured as an input with internal pulled-down
    GPIO_setAsInputPinWithPullDownResistor(WP_PORT, WP_BIT);

    // Check the current state of the write-protect pin
    if(GPIO_getInputPinValue(WP_PORT, WP_BIT))
    {
        // This indicates the pin state is high - write protected state

        // Configure WP pin as high to low transition
        GPIO_selectInterruptEdge(WP_PORT, WP_BIT, GPIO_HIGH_TO_LOW_TRANSITION);

        // Set the write protect bit
        writeProtected = 1;
    }
    else
    {
        // This indicates the pin state is low - write enabled state

        // Configure WP pin as high to low transition
        GPIO_selectInterruptEdge(WP_PORT, WP_BIT, GPIO_LOW_TO_HIGH_TRANSITION);

        // Clear the write protect bit
        writeProtected = 0;
    }

    // Clear the WP pin interrupt flag
    GPIO_clearInterrupt(WP_PORT, WP_BIT);

    // Enable the WP pin interrupt
    GPIO_enableInterrupt(WP_PORT, WP_BIT);

    // Initialize the serial communication interface
#if defined(INTERFACE_I2C)
    eeprom_init_I2cInterface();
#elif defined (INTERFACE_SPI)
    eeprom_init_SpiInterface();
#else
#error "No interface defined"
#endif
}

void eeprom_reEnableInterface(void)
{
#if defined(INTERFACE_I2C)
    eeprom_reEnableI2c();
#elif defined (INTERFACE_SPI)
    eeprom_reEnableSpi();
#else
#error "No interface defined"
#endif
}

void eeprom_disableInterface(void)
{
#if defined(INTERFACE_I2C)
    eeprom_disableI2c();
#elif defined (INTERFACE_SPI)
    eeprom_disableSpi();
#else
#error "No interface defined"
#endif
}

#ifdef USE_DMA
inline void eeprom_getMemoryWithDma(uint32_t index)
{
    // Check if the index is first valid
    if(index > eeprom_total_size)
    {
        // Reset back the eeprom pointer back to the start address
        eepromReadDataPointerAddress = eeprom1_start_address;
    }
    else
    {
        if(index >= EEPROM1_SIZE)
        {
            // This means that the pointer should be at the bottom memory segment
#if defined(__TI_COMPILER_VERSION__)
#pragma diag_suppress 770
#endif
            // Source block address
            __data16_write_addr((uint16_t) &DMA3SA,
                                ((uint32_t) eeprom2_start_address + index -
                                 EEPROM1_SIZE));

            // Destination single address
            __data16_write_addr((uint16_t) &DMA3DA,
                                (uint32_t) &HWREG16(EEPROM_SPI_EUSCI_BASE +
                                                    OFS_UCBxTXBUF));
#if defined(__TI_COMPILER_VERSION__)
#pragma diag_warning 770
#endif

            // Configure the DMA trigger source
            DMACTL1 = DMA3TSEL__UCB1TXIFG0;

            // Specify the maximum allowable writable size
            // Calculate to ensure the destination address is not going to overflow the memory that is available
            uint32_t lastAddressSize = (EEPROM2_SIZE - (index - EEPROM1_SIZE));
            if((lastAddressSize) < 0xFFFF)
            {
                DMA3SZ = lastAddressSize + 1;
            }
            else
            {
                DMA3SZ = 0xFFFF;
            }
            dmaSize = DMA3SZ;

            DMA3CTL = DMADT_1 | // Block transfer
                      DMASRCINCR_3 | // source address is incremented
                      DMADSTINCR_0 | // Destination address is unchanged
                      DMADSTBYTE | // Destination Byte increment
                      DMASRCBYTE | // Source Byte increment
                      DMALEVEL | // Use DMA level
                      DMAIE |   // Enable interrupt
                      DMAEN |   // Enable DMA3
                      DMAREQ;   // Trigger DMA
        }
        else
        {
            // This means that the pointer should be at the bottom memory segment
#if defined(__TI_COMPILER_VERSION__)
#pragma diag_suppress 770
#endif
            // Source block address
            __data16_write_addr((uint16_t) &DMA3SA,
                                ((uint32_t) eeprom1_start_address + index));

            // Destination single address
            __data16_write_addr((uint16_t) &DMA3DA,
                                (uint32_t) &HWREG16(EEPROM_SPI_EUSCI_BASE +
                                                    OFS_UCBxTXBUF));
#if defined(__TI_COMPILER_VERSION__)
#pragma diag_warning 770
#endif

            // Configure the DMA trigger source
            DMACTL1 = DMA3TSEL__UCB1TXIFG0;

            // Specify the maximum allowable writable size
            // Since the lower memory segment is less than 64k, we assume
            // it will be a continuous transmission
            DMA3SZ = (uint16_t) (EEPROM1_SIZE - index);

            DMA3CTL = DMADT_1 | // Block transfer
                      DMASRCINCR_3 | // source address is incremented
                      DMADSTINCR_0 | // Destination address is unchanged
                      DMADSTBYTE | // Destination Byte increment
                      DMASRCBYTE | // Source Byte increment
                      DMALEVEL | // Use DMA level
                      DMAIE |  // Enable interrupt
                      DMAEN |  // Enable DMA3
                      DMAREQ;  // Trigger DMA
        }
    }
}

#endif

inline uint8_t eeprom_getMemory(uint32_t index)
{
    // Check if the index is first valid
    if(index > eeprom_total_size)
    {
        // Reset back the eeprom pointer back to the start address
        eepromReadDataPointerAddress = eeprom1_start_address;
    }
    else
    {
        if(index >= EEPROM1_SIZE)
        {
#ifdef EEPROM2_SIZE
            // This means that the pointer should be at the top memory segment
            eepromReadDataPointerAddress =
                (uint8_t *) ((uint32_t) eeprom2_start_address + index -
                             EEPROM1_SIZE);
#endif
        }
        else
        {
            // This means that the pointer should be at the bottom memory segment
            eepromReadDataPointerAddress =
                (uint8_t *) ((uint32_t) eeprom1_start_address + index);
        }
    }
    return(*eepromReadDataPointerAddress);
}

inline void eeprom_writeMemory(uint32_t index,
                               uint8_t value)
{
    // Only write to memory if write is enabled
    if(!writeProtected)
    {
#ifdef USE_DMA
        // Check if the index is first valid
        if(index > eeprom_total_size)
        {
            // Reset back the write data pointer address to the start
            eepromWriteDataPointerAddress = eeprom1_start_address;
            // and don't write anything to memory
        }
        else
        {
            if(index >= EEPROM1_SIZE)
            {
#if defined(__TI_COMPILER_VERSION__)
#pragma diag_suppress 770
#endif
                // Configure DMA3 to start receiving
                // This means that the pointer should be at the top memory segment
                // Use DMA3 for upper memory write
                //
                // Configure DMA channel 0 to transmit
                // Source block address
                __data16_write_addr((uint16_t) &DMA3SA,
                                    (uint32_t) &HWREG16(EEPROM_SPI_EUSCI_BASE +
                                                        OFS_UCBxRXBUF));

                // Destination single address
                __data16_write_addr((uint16_t) &DMA3DA,
                                    ((uint32_t) eeprom2_start_address + index -
                                     EEPROM1_SIZE));
#if defined(__TI_COMPILER_VERSION__)
#pragma diag_warning 770
#endif

                // Configure the DMA trigger source
                DMACTL1 = DMA3TSEL__UCB1RXIFG0;

                // Specify the maximum allowable writable size
                // Calculate to ensure the destination address is not going to overflow the memory that is available
                uint32_t lastAddressSize =
                    (EEPROM2_SIZE - (index - EEPROM1_SIZE));
                if((lastAddressSize) < 0xFFFF)
                {
                    DMA3SZ = lastAddressSize + 1;
                }
                else
                {
                    DMA3SZ = 0xFFFF;
                }
                dmaSize = DMA3SZ;

                DMA3CTL = DMADT_1 | // Block transfer
                          DMASRCINCR_0 | // source address is unchanged
                          DMADSTINCR_3 | // Destination address is incremented
                          DMADSTBYTE | // Destination Byte increment
                          DMASRCBYTE | // Source Byte increment
                          DMALEVEL | // Use DMA level
                          DMAIE |   // Enable interrupt
                          DMAEN |   // Enable DMA3
                          DMAREQ;   // Trigger DMA
            }
            else
            {
                // Use DMA3 as a dedicated DMA for lower memory write
                // since it is lower than 64kb size
#if defined(__TI_COMPILER_VERSION__)
#pragma diag_suppress 770
#endif
                // Source block address
                __data16_write_addr((uint16_t) &DMA3SA,
                                    (uint32_t) &HWREG16(EEPROM_SPI_EUSCI_BASE +
                                                        OFS_UCBxRXBUF));

                // Destination single address
                __data16_write_addr((uint16_t) &DMA3DA,
                                    ((uint32_t) eeprom1_start_address + index));
#if defined(__TI_COMPILER_VERSION__)
#pragma diag_warning 770
#endif
                // Configure the DMA trigger source
                DMACTL1 = DMA3TSEL__UCB1RXIFG0;

                // Specify the maximum allowable writable size
                // Since the lower memory segment is less than 64k, we assume
                // it will be a continuous transmission
                DMA3SZ = (uint16_t) (EEPROM1_SIZE - index);

                DMA3CTL = DMADT_1 | // Block transfer
                          DMASRCINCR_0 | // source address is unchanged
                          DMADSTINCR_3 | // Destination address is incremented
                          DMADSTBYTE | // Destination Byte increment
                          DMASRCBYTE | // Source Byte increment
                          DMALEVEL | // Use DMA level
                          DMAIE |   // Enable interrupt
                          DMAEN |   // Enable DMA3
                          DMAREQ;   // Trigger DMA
            }
        }
#else
        // Check if the index is first valid
        if(index > eeprom_total_size)
        {
            // Reset back the write data pointer address to the start
            eepromWriteDataPointerAddress = eeprom1_start_address;
            // and don't write anything to memory
        }
        else
        {
            if(index >= EEPROM1_SIZE)
            {
#ifdef EEPROM2_SIZE
                // This means that the pointer should be at the top memory segment
                eepromWriteDataPointerAddress =
                    (uint8_t *) ((uint32_t) eeprom2_start_address + index -
                                 EEPROM1_SIZE);
#endif
            }
            else
            {
                // This means that the pointer should be at the bottom memory segment
                eepromWriteDataPointerAddress =
                    (uint8_t *) ((uint32_t) eeprom1_start_address + index);
            }
            // Write to FRAM memory
            *eepromWriteDataPointerAddress = value;
        }
#endif
    }
}

//******************************************************************************
//
//! \brief  Initializes all the eeprom memory bounds
//!
//! This API initializes all the EEPROM end addresses for the lower and upper
//! memory bound. It also initializes the current Read and Write data pointers.
//!
//! \param  none
//!
//! \return none
//
//******************************************************************************
void eeprom_initializeMemoryBound(void)
{
    // Initializes the Read and Write data pointer address to the start of
    // EEPROM memory
    eepromReadDataPointerAddress = (uint8_t *) eeprom1_start_address;
    eepromWriteDataPointerAddress = (uint8_t *) eeprom1_start_address;
}

//******************************************************************************
//
//! \brief  Handles the Write Protect pin when an interrupt fires
//!
//! This function needs to be called within the GPIO pin interrupt service
//! handler.
//!
//! \param  none
//!
//! \return none
//
//******************************************************************************
void eeprom_writeProtectInterrupt(void)
{
    if(WP_IES & WP_BIT)
    {
        // Means a high-to-low transition occurred
        // Configure WP pin as high to low transition
        WP_IES &= ~WP_BIT;

        // Clear the write protect bit. Write is now enabled.
        writeProtected = 0;
    }
    else
    {
        // Means a low-to-high transition occurred

        // Configure WP pin as high to low transition
        WP_IES |= WP_BIT;

        // Set the write protect bit. Write is now protected.
        writeProtected = 1;
    }
}

#ifdef USE_DMA
//******************************************************************************
//
//! \brief  Disables the DMA and updates the write index pointer
//!
//! Disables the DMA
//!
//! \param  none
//!
//! \return none
//
//******************************************************************************
void eeprom_disableDma(void)
{
    uint32_t size;

    // Set the new writeindexpointer
    size = EEPROM1_SIZE - DMA3SZ;
    eepromWriteIndexPointer = eepromWriteIndexPointer + size;

    // Disable the DMA
    DMA3CTL &= ~DMAEN;
}

//------------------------------------------------------------------------------
// DMA interrupt handler is used to received data when it reaches the maximum
// size that is defined in the DMAxSZ and updates the necessary pointers
// and restarts the DMA again if there is more data.
//------------------------------------------------------------------------------
#pragma vector=DMA_VECTOR
__interrupt void dma_isr_handler(void)
{
    switch(__even_in_range(DMAIV, DMAIV__DMA5IFG))
    {
    case DMAIV__NONE:       break;
    case DMAIV__DMA0IFG:    break;
    case DMAIV__DMA1IFG:    break;
    case DMAIV__DMA2IFG:    break;
    case DMAIV__DMA3IFG:
        if(spiOpCode == OPCODE_WRITE)
        {
            uint32_t size = DMA3SZ;
            // Reconfigured to write at either the upper or lower
            // memory bound
            eepromWriteIndexPointer = eepromWriteIndexPointer + size;
            eeprom_writeMemory(eepromWriteIndexPointer, 0);
        }
        else if(spiOpCode == OPCODE_READ)
        {
            uint32_t size = DMA3SZ;
            // Reconfigured to write at either the upper or lower
            // memory bound
            eepromReadIndexPointer = eepromReadIndexPointer + size;
            eeprom_getMemoryWithDma(eepromReadIndexPointer);
        }
        break;
    case DMAIV__DMA4IFG:    break;
    case DMAIV__DMA5IFG:    break;
    default: break;
    }
}

#endif
