/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#ifndef EEPROM_H_
#define EEPROM_H_

//******************************************************************************
//
//! \brief  Initializes the interface used by EEPROM as slave mode
//!
//! This API initializes the serial communication interface in slave mode which
//! could be I2C or SPI mode. The interface can be selected in
//! eeprom_definitions.h header file.
//!
//! \param  none
//!
//! \return none
//
//******************************************************************************
extern void eeprom_initializeInterface(void);

//******************************************************************************
//
//! \brief  Re-Initializes the interface module to start receiving again
//!
//! This API re-initializes the serial interface. This is used mainly for I2C
//! communication as it needs to generate a NACK.
//!
//! \param  none
//!
//! \return none
//
//******************************************************************************
extern void eeprom_reEnableInterface(void);

//******************************************************************************
//
//! \brief  Disables the interface module by putting it into reset mode
//!
//! This API is mainly used to generate a NACK for I2C
//! while the device is still busy writing to FRAM memory during EEPROM
//! operation.
//!
//! \param  none
//!
//! \return none
//
//******************************************************************************
extern void eeprom_disableInterface(void);

//******************************************************************************
//
//! \brief  Sends serial data using DMA based on the index pointer
//!
//! This API is different from eeprom_getMemory() as this uses DMA to transfer
//! all data by using the DMA.
//!
//! \param  index   EEPROM memory address
//!
//! \return Stored EEPROM value based on the index
//
//******************************************************************************
extern inline void eeprom_getMemoryWithDma(uint32_t index);

//******************************************************************************
//
//! \brief  Returns the value from memory based on the EEPROM index address
//!
//! This API handles all the necessary pointers and returns the value of the
//! index that is passed in.
//!
//! \param  index   EEPROM memory address
//!
//! \return Stored EEPROM value based on the index
//
//******************************************************************************
extern uint8_t eeprom_getMemory(uint32_t index);

//******************************************************************************
//
//! \brief  Writes a value to EEPROM memory based on the index address
//!
//! This API handles all the necessary pointers and writes the value to the
//! index that is passed in.
//!
//! \param  index   EEPROM memory address
//! \param  value   Value to be written
//!
//! \return none
//
//******************************************************************************
extern void eeprom_writeMemory(uint32_t index,
                               uint8_t value);

//******************************************************************************
//
//! \brief  Handles the Write Protect pin when an interrupt fires
//!
//! This function needs to be called within the GPIO pin interrupt service
//! handler.
//!
//! \param  none
//!
//! \return none
//
//******************************************************************************
extern void eeprom_writeProtectInterrupt(void);

//******************************************************************************
//
//! \brief  Disables the DMA and updates the write index pointer
//!
//! Disables the DMA
//!
//! \param  none
//!
//! \return none
//
//******************************************************************************
extern void eeprom_disableDma(void);

#endif
