/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#include <msp430.h>
#include <stdint.h>
#include <string.h>
#include <driverlib.h>
#include "eeprom.h"
#include "eeprom_i2c.h"
#include "eeprom_definitions.h"
#include "sensing_proc.h"

extern uint8_t eepromRxData[];
extern uint8_t eepromRxDataPointer;
extern uint8_t lpmState;

static uint8_t i2cTxFlag = 0;
static uint8_t sensorHeaderProtocol;
static uint8_t * sensorDataTx;
static uint8_t sensorDataTxSize;
static uint16_t receivedPointer;

/*
 * Initializes the I2C interface as Slave communication
 */
void eeprom_init_I2cInterface(void)
{
    GPIO_setAsPeripheralModuleFunctionInputPin(
        EEPROM_SLAVE_SDA_SCL_PORT,
        EEPROM_SLAVE_SDA_PIN + EEPROM_SLAVE_SCL_PIN,
        GPIO_PRIMARY_MODULE_FUNCTION
        );

    // eUSCI configuration
    EUSCI_B_I2C_initSlaveParam param;
    param.slaveAddress = EEPROM_MEMORY_SLAVE_ADDRESS;
    param.slaveAddressOffset = EUSCI_B_I2C_OWN_ADDRESS_OFFSET0;
    param.slaveOwnAddressEnable = EUSCI_B_I2C_OWN_ADDRESS_ENABLE;
    EUSCI_B_I2C_initSlave(EEPROM_I2C_EUSCI_BASE, &param);

#if defined(EEPROM_COPRO_SLAVE_ADDRESS)
    // Enable multi-slave addressing mode
    UCB1I2COA1 = UCOAEN | EEPROM_COPRO_SLAVE_ADDRESS;
#endif

    // Enable the I2C module
    EUSCI_B_I2C_enable(EEPROM_I2C_EUSCI_BASE);

    EUSCI_B_I2C_clearInterrupt(EEPROM_I2C_EUSCI_BASE,
                               EUSCI_B_I2C_RECEIVE_INTERRUPT0 |
                               EUSCI_B_I2C_TRANSMIT_INTERRUPT0 |
#if defined(EEPROM_COPRO_SLAVE_ADDRESS)
                               EUSCI_B_I2C_RECEIVE_INTERRUPT1 |
                               EUSCI_B_I2C_TRANSMIT_INTERRUPT1 |
#endif
                               EUSCI_B_I2C_START_INTERRUPT |
                               EUSCI_B_I2C_STOP_INTERRUPT
                               );

    EUSCI_B_I2C_enableInterrupt(EEPROM_I2C_EUSCI_BASE,
                                EUSCI_B_I2C_RECEIVE_INTERRUPT0 |
                                EUSCI_B_I2C_TRANSMIT_INTERRUPT0 |
#if defined(EEPROM_COPRO_SLAVE_ADDRESS)
                                EUSCI_B_I2C_RECEIVE_INTERRUPT1 |
                                EUSCI_B_I2C_TRANSMIT_INTERRUPT1 |
#endif
                                EUSCI_B_I2C_START_INTERRUPT |
                                EUSCI_B_I2C_STOP_INTERRUPT
                                );

    // Resets the flag for receive pointer
    receivedPointer = 0;
}

/*
 * Initializes the I2C interface as Slave communication
 */
void eeprom_reEnableI2c(void)
{
    // Clears the flag that the received pointer can receive again
    receivedPointer = 0;

    EUSCI_B_I2C_enable(EEPROM_I2C_EUSCI_BASE);

    EUSCI_B_I2C_clearInterrupt(EEPROM_I2C_EUSCI_BASE,
                               EUSCI_B_I2C_RECEIVE_INTERRUPT0 |
                               EUSCI_B_I2C_TRANSMIT_INTERRUPT0 |
#if defined(EEPROM_COPRO_SLAVE_ADDRESS)
                               EUSCI_B_I2C_RECEIVE_INTERRUPT1 |
                               EUSCI_B_I2C_TRANSMIT_INTERRUPT1 |
#endif
                               EUSCI_B_I2C_START_INTERRUPT |
                               EUSCI_B_I2C_STOP_INTERRUPT
                               );

    EUSCI_B_I2C_enableInterrupt(EEPROM_I2C_EUSCI_BASE,
                                EUSCI_B_I2C_RECEIVE_INTERRUPT0 |
                                EUSCI_B_I2C_TRANSMIT_INTERRUPT0 |
#if defined(EEPROM_COPRO_SLAVE_ADDRESS)
                                EUSCI_B_I2C_RECEIVE_INTERRUPT1 |
                                EUSCI_B_I2C_TRANSMIT_INTERRUPT1 |
#endif
                                EUSCI_B_I2C_START_INTERRUPT |
                                EUSCI_B_I2C_STOP_INTERRUPT
                                );
}

/*
 * Disables the I2C interface as Slave communication
 */
void eeprom_disableI2c(void)
{
    EUSCI_B_I2C_disable(EEPROM_I2C_EUSCI_BASE);
}

//------------------------------------------------------------------------------
// The USCI_B1 data ISR RX vector is used to move received data from the I2C
// master to the MSP430 memory.
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
// The USCI_B1 state ISR TX vector is used to wake up the CPU from LPM0 in order
// to process the received data in the main program. LPM0 is only exit in case
// of a (re-)start or stop condition when actual data was received.
//------------------------------------------------------------------------------
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=USCI_xx_VECTOR
__interrupt
#elif defined(__GNUC__)
__attribute__((interrupt(USCI_xx_VECTOR)))
#endif
void USCIBx_ISR(void)
{
    switch(__even_in_range(HWREG16(EEPROM_I2C_EUSCI_BASE + OFS_UCBxIV),
                           USCI_I2C_UCBIT9IFG))
    {
    case USCI_NONE:          break;         // Vector 0: No interrupts
    case USCI_I2C_UCALIFG:   break;         // Vector 2: ALIFG
    case USCI_I2C_UCNACKIFG: break;         // Vector 4: NACKIFG
    case USCI_I2C_UCSTTIFG:                 // Vector 6: STTIFG
        // Set the state to LPM0
        lpmState = 0;

        // Switch to LPM0 from LPM3 on starting the data transmission
        __bic_SR_register_on_exit(SCG1 | SCG0);

        // If the eepromRxDataPointer != 0, it means that it received
        // another start sequence. This is when a Random/Sequential read
        // happens.
        // Hence, it needs to handle repeated start
        if(eepromRxDataPointer != 0)
        {
#if (EEPROM_ADDRESS_BYTE_SIZE == 2)
            // Store the new read index pointer for 2 byte address
            eepromReadIndexPointer =
                (uint32_t) ((uint16_t) (eepromRxData[0] << 8) +
                            (uint8_t) eepromRxData[1]);
#elif (EEPROM_ADDRESS_BYTE_SIZE == 3)
            // Store the new read index pointer for 3 byte address
            eepromReadIndexPointer =
                (uint32_t) ((((uint32_t) eepromRxData[0]) << 16) +
                            (((uint32_t) eepromRxData[1]) << 8) +
                            ((uint32_t) eepromRxData[2]));
#endif
            // Reset the i2c buffer data pointer
            eepromRxDataPointer = 0;
        }

        break;
    case USCI_I2C_UCSTPIFG:                 // Vector 8: STPIFG

        // Check if the EEPROM was transmitting data and then receives a
        // stop command
        if(i2cTxFlag)
        {
            // Decrement the pointer by 1 since TXBUF is double
            // buffered and has post-incremented the ReadIndexPointer.
            // This is mainly used for Current Address Read and Random Read
            // functionality of an EEPROM.
            eepromReadIndexPointer--;
            i2cTxFlag = 0;
        }

        // Reset the i2c buffer data pointer
        eepromRxDataPointer = 0;
        receivedPointer = 0;

        // Set the state back to LPM3
        lpmState = 1;

        // Switch back to LPM3 from LPM0 on powering down
        __bis_SR_register_on_exit(SCG1 | SCG0);
        break;
    case USCI_I2C_UCRXIFG3: break;          // Vector 10: RXIFG3
    case USCI_I2C_UCTXIFG3: break;          // Vector 14: TXIFG3
    case USCI_I2C_UCRXIFG2: break;          // Vector 16: RXIFG2
    case USCI_I2C_UCTXIFG2: break;          // Vector 18: TXIFG2
    case USCI_I2C_UCRXIFG1:                 // Vector 20: RXIFG1

        // Receive and check which protocol received
        sensorHeaderProtocol = EUSCI_B_I2C_slaveGetData(EEPROM_I2C_EUSCI_BASE);
        switch(sensorHeaderProtocol)
        {
        // Set the pointer to the appropriate array to be transmitted
        case SENSOR_HEADER_VCC:
            sensorDataTx = currentVccArray;
            sensorDataTxSize = 4;
            break;
        case SENSOR_HEADER_TEMP_C:
            sensorDataTx = currentTemperatureDegCArray;
            sensorDataTxSize = 4;
            break;
        case SENSOR_HEADER_TEMP_F:
            sensorDataTx = currentTemperatureDegFArray;
            sensorDataTxSize = 4;
            break;
        default: break;
        }
        break;
    case USCI_I2C_UCTXIFG1:                 // Vector 22: TXIFG1
        // Transmit the data out to master
        EUSCI_B_I2C_slavePutData(EEPROM_I2C_EUSCI_BASE,
                                 *sensorDataTx);
        sensorDataTx++;
        sensorDataTxSize--;
        break;
    case USCI_I2C_UCRXIFG0:
#if defined(DEBUG_EEPROM)
        // Set P1.0 for debugging purposes
        P1OUT |= BIT0;
#endif
        // Get the initial index address
        if(!receivedPointer)
        {
            // Master is writing data. Slave reads and puts into buffer first
            eepromRxData[eepromRxDataPointer] =
                EUSCI_B_I2C_slaveGetData(EEPROM_I2C_EUSCI_BASE);

            // If the buffer is reached, set the new pointer address
            if(eepromRxDataPointer == (EEPROM_ADDRESS_BYTE_SIZE - 1))
            {
#if (EEPROM_ADDRESS_BYTE_SIZE == 2)
                // get the pointer address for 2 byte address
                eepromWriteIndexPointer =
                    (uint32_t) ((uint16_t) (eepromRxData[0] << 8) +
                                (uint8_t) eepromRxData[1]);
#elif (EEPROM_ADDRESS_BYTE_SIZE == 3)
                // get the pointer address for 3 byte address
                eepromWriteIndexPointer =
                    (uint32_t) ((((uint32_t) eepromRxData[0]) << 16) +
                                (((uint32_t) eepromRxData[1]) << 8) +
                                ((uint32_t) eepromRxData[2]));
#endif
                // set a flag indicating that the address is received
                receivedPointer = 1;
            }
            eepromRxDataPointer++;
        }
        else
        {
            // Write the value to the index pointer register
            eeprom_writeMemory(eepromWriteIndexPointer++,
                               EUSCI_B_I2C_slaveGetData(EEPROM_I2C_EUSCI_BASE));
        }
#if defined(DEBUG_EEPROM)
        // Clear P1.0 for debugging purposes
        P1OUT &= ~BIT0;
#endif
        break;                              // Vector 24: RXIFG0
    case USCI_I2C_UCTXIFG0:                 // Vector 26: TXIFG0
#if defined(DEBUG_EEPROM)
        // Set P1.0 for debugging purposes
        P4OUT |= BIT0;
#endif
        EUSCI_B_I2C_slavePutData(EEPROM_I2C_EUSCI_BASE,
                                 eeprom_getMemory(eepromReadIndexPointer++));
        i2cTxFlag = 1;
#if defined(DEBUG_EEPROM)
        // Clear P1.0 for debugging purposes
        P4OUT &= ~BIT0;
#endif
        break;
    case USCI_I2C_UCBCNTIFG: break;         // Vector 28: BCNTIFG
    case USCI_I2C_UCCLTOIFG: break;         // Vector 30: clock low timeout
    case USCI_I2C_UCBIT9IFG: break;         // Vector 32: 9th bit
    default: break;
    }
}

//------------------------------------------------------------------------------
// Port 8 interrupt service routine. This example uses P8.3 for
// write protection bit.
//------------------------------------------------------------------------------
#pragma vector=PORT8_VECTOR
__interrupt void port8_isr_handler(void)
{
    switch(__even_in_range(P8IV, P8IV_P8IFG7))
    {
    case P8IV_NONE:    break;               // Vector  0:  No interrupt
    case P8IV_P8IFG0:  break;               // Vector  2:  P8.0 IFG
    case P8IV_P8IFG1:  break;               // Vector  4:  P8.1 IFG
    case P8IV_P8IFG2:  break;               // Vector  6:  P8.2 IFG
    case P8IV_P8IFG3:                       // Vector  8:  P8.3 IFG
        eeprom_writeProtectInterrupt();
        break;
    case P8IV_P8IFG4:  break;               // Vector  10: P8.4 IFG
    case P8IV_P8IFG5:  break;               // Vector  12: P8.5 IFG
    case P8IV_P8IFG6:  break;               // Vector  14: P8.6 IFG
    case P8IV_P8IFG7:  break;               // Vector  14: P8.7 IFG
    default: break;
    }
}
