/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#include <msp430.h>
#include <stdint.h>
#include <string.h>
#include <driverlib.h>
#include "eeprom.h"
#include "eeprom_spi.h"
#include "eeprom_definitions.h"
#include "sensing_proc.h"

#ifdef USE_DMA
#ifndef __MSP430_HAS_DMA__
#error "This device does not have DMA."
#endif
#endif

extern uint8_t eepromRxData[];
extern uint8_t eepromRxDataPointer;
extern uint8_t lpmState;

#if defined(EEPROM_COPRO_SLAVE_ADDRESS)
static uint8_t sensorHeaderProtocol;
static uint8_t * sensorDataTx;
static uint8_t sensorDataTxSize;
#endif
uint8_t spiOpCode;
static uint16_t receivedPointer;

static const EUSCI_B_SPI_initSlaveParam param = {
    EUSCI_B_SPI_MSB_FIRST,
    EUSCI_B_SPI_PHASE_DATA_CHANGED_ONFIRST_CAPTURED_ON_NEXT,
    EUSCI_B_SPI_CLOCKPOLARITY_INACTIVITY_HIGH,
    EUSCI_B_SPI_3PIN
};
/*
 * Initializes the SPI interface as Slave communication
 */
void eeprom_init_SpiInterface(void)
{
    // Configure SPI MOSI and MISO pins
    GPIO_setAsPeripheralModuleFunctionInputPin(
        EEPROM_SLAVE_MOSI_MISO_PORT,
        EEPROM_SLAVE_MISO_PIN | EEPROM_SLAVE_MOSI_PIN,
        GPIO_PRIMARY_MODULE_FUNCTION);

    // Configure SPI SCLK pin
    GPIO_setAsPeripheralModuleFunctionInputPin(
        EEPROM_SLAVE_SCLK_PORT,
        EEPROM_SLAVE_SCLK_PIN,
        GPIO_PRIMARY_MODULE_FUNCTION);

    // Configure CSn pin with internal pulled-up
    GPIO_setAsInputPinWithPullUpResistor(
        EEPROM_SLAVE_CSn_PORT,
        EEPROM_SLAVE_CSn_PIN);

    // Configure CSn pin with interrupt enable on high-to-low transition
    GPIO_selectInterruptEdge(
        EEPROM_SLAVE_CSn_PORT,
        EEPROM_SLAVE_CSn_PIN,
        GPIO_HIGH_TO_LOW_TRANSITION);
    GPIO_clearInterrupt(
        EEPROM_SLAVE_CSn_PORT,
        EEPROM_SLAVE_CSn_PIN);
    GPIO_enableInterrupt(
        EEPROM_SLAVE_CSn_PORT,
        EEPROM_SLAVE_CSn_PIN);

    // eUSCI configuration
    EUSCI_B_SPI_initSlave(EEPROM_SPI_EUSCI_BASE,
                          (EUSCI_B_SPI_initSlaveParam *) &param);

    // Resets the flag for receive pointer
    receivedPointer = 0;
}

/*
 * Initializes the SPI interface as Slave communication
 */
void eeprom_reEnableSpi(void)
{
    // This line of code has been hand optimized for timing reasons

    // Clears the flag that the received pointer can receive again
    receivedPointer = 0;

    // Enable the SPI module
    HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxCTLW0) &= ~(UCSWRST);

    // Clear the receive interrupt flag and enable interrupts
    HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxIFG) &= ~(UCRXIFG);
    HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxIE) |= (UCRXIE);
}

/*
 * Disables the SPI interface as Slave communication
 */
void eeprom_disableSpi(void)
{
    // Disable the SPI module
    HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxCTLW0) |= (UCSWRST);
}

//------------------------------------------------------------------------------
// The USCI_xx ISR is used to check the necessary protocol and either reads or
// trasnfers data back to the host processor. This is a critical timing piece
// of software that is tuned to work with certain specification. Please see
// the TI-Design for the timing specification.
//------------------------------------------------------------------------------
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=USCI_xx_VECTOR
__interrupt
#elif defined(__GNUC__)
__attribute__((interrupt(USCI_xx_VECTOR)))
#endif
void USCIBx_ISR(void)
{
    switch(__even_in_range(HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxIV),
                           USCI_SPI_UCTXIFG))
    {
    case USCI_NONE:         break;          // Vector 0: No interrupts
    case USCI_SPI_UCRXIFG:                  // Vector 2: UCRXIFG
#if defined(DEBUG_EEPROM)
        // Set P1.0 for debugging purposes
        P1OUT |= BIT0;
#endif
        if(spiOpCode == 0)
        {
            // Get the data packets. First data packet is always the Op Code
            spiOpCode = HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxRXBUF);
        }
        else
        {
            switch(spiOpCode)
            {
            case OPCODE_WRITE:
                // Get the initial index address
                if(!receivedPointer)
                {
                    // Master is writing data. Slave reads and puts into buffer first
                    eepromRxData[eepromRxDataPointer] =
                        HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxRXBUF);

                    // If the buffer is reached, set the new pointer address
                    if(eepromRxDataPointer == (EEPROM_ADDRESS_BYTE_SIZE - 1))
                    {
#if (EEPROM_ADDRESS_BYTE_SIZE == 2)
                        // get the pointer address for 2 byte address
                        eepromWriteIndexPointer =
                            (uint32_t) ((((uint32_t) eepromRxData[0]) << 8) +
                                         ((uint32_t) eepromRxData[1]));
#elif (EEPROM_ADDRESS_BYTE_SIZE == 3)
                        // get the pointer address for 3 byte address
                        eepromWriteIndexPointer =
                            (uint32_t) ((((uint32_t) eepromRxData[0]) << 16) +
                                        (((uint32_t) eepromRxData[1]) << 8) +
                                        ((uint32_t) eepromRxData[2]));
#endif
                        // set a flag indicating that the address is received
                        receivedPointer = 1;

#ifdef USE_DMA
                        // Disable receive interrupt enable
                        HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxIE) &=
                            ~UCRXIE;

                        // Write the value to the index pointer register
                        // this triggers the DMA
                        eeprom_writeMemory(eepromWriteIndexPointer, 0);
#endif
                    }
                    eepromRxDataPointer++;
                }
                else
                {
#ifndef USE_DMA
                    // Write the value to the index pointer register
                    eeprom_writeMemory(eepromWriteIndexPointer++,
                                       HWREG16(EEPROM_SPI_EUSCI_BASE +
                                               OFS_UCBxRXBUF));
#endif
                }
                break;
            case OPCODE_READ:
                // Check if the address has been received first
                if(!receivedPointer)
                {
                    // Master is writing data. Slave reads and puts into buffer first
                    eepromRxData[eepromRxDataPointer] =
                        HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxRXBUF);

                    // If the buffer is reached, set the new pointer address
                    if(eepromRxDataPointer == (EEPROM_ADDRESS_BYTE_SIZE - 1))
                    {
#if (EEPROM_ADDRESS_BYTE_SIZE == 2)
                        // get the pointer address for 2 byte address
                        eepromReadIndexPointer =
                           (uint32_t) ((((uint32_t) eepromRxData[0]) << 8) +
                                        ((uint32_t) eepromRxData[1]));
#elif (EEPROM_ADDRESS_BYTE_SIZE == 3)
                        // get the pointer address for 3 byte address
                        eepromReadIndexPointer =
                            (uint32_t) ((((uint32_t) eepromRxData[0]) << 16) +
                                        (((uint32_t) eepromRxData[1]) << 8) +
                                        ((uint32_t) eepromRxData[2]));
#endif
                        // set a flag indicating that the address is received
                        receivedPointer = 1;

                        // Disable receive interrupt enable since
                        // the application is using DMA
                        HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxIE) &= ~UCRXIE;
#ifdef USE_DMA
                        // Preload the data here
                        HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxTXBUF) =
                            eeprom_getMemory(eepromReadIndexPointer++);

                        // Use DMA to transmit data directly
                        eeprom_getMemoryWithDma(eepromReadIndexPointer);
#else
                        // Enable transmit interrupt
                        HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxIE) |= UCTXIE;

                        // Preload the data here
                        HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxTXBUF) =
                            eeprom_getMemory(eepromReadIndexPointer++);
#endif
                    }
                    eepromRxDataPointer++;
                }
                break;
            case OPCODE_SENSOR:
                // Master is writing data. Slave reads and puts into buffer first
                if(sensorHeaderProtocol == 0)
                {
                    sensorHeaderProtocol =
                        HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxRXBUF);
                    switch(sensorHeaderProtocol)
                    {
                    // Set the transfer pointer
                    case SENSOR_HEADER_VCC:
                        sensorDataTx = currentVccArray;
                        sensorDataTxSize = 4;
                        break;
                    case SENSOR_HEADER_TEMP_C:
                        sensorDataTx = currentTemperatureDegCArray;
                        sensorDataTxSize = 4;
                        break;
                    case SENSOR_HEADER_TEMP_F:
                        sensorDataTx = currentTemperatureDegFArray;
                        sensorDataTxSize = 4;
                        break;
                    default: break;
                    }

#ifdef USE_DMA
                    // Disable receive interrupt enable since
                    // the application is using DMA
                    HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxIE) &= ~UCRXIE;

                    // Preload the buffer for the first byte
                    HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxTXBUF) =
                        *sensorDataTx;
                    sensorDataTx++;
                    sensorDataTxSize--;
#if defined(__TI_COMPILER_VERSION__)
#pragma diag_suppress 770
#endif
                    // Source block address
                    __data16_write_addr((uint16_t) &DMA3SA,
                                        ((uint32_t) sensorDataTx));

                    // Destination single address
                    __data16_write_addr((uint16_t) &DMA3DA,
                                        (uint32_t) &HWREG16(
                                            EEPROM_SPI_EUSCI_BASE +
                                            OFS_UCBxTXBUF));
#if defined(__TI_COMPILER_VERSION__)
#pragma diag_warning 770
#endif
                    // Configure the DMA trigger source
                    DMACTL1 = DMA3TSEL__UCB1TXIFG0;

                    // Set buffer size left to transfer after
                    // preloading the buffer from above
                    DMA3SZ = sensorDataTxSize;

                    DMA3CTL = DMADT_1 |         // Block transfer
                              DMASRCINCR_3 |    // source address is incremented
                              DMADSTINCR_0 |    // Destination address is unchanged
                              DMADSTBYTE |      // Destination Byte increment
                              DMASRCBYTE |      // Source Byte increment
                              DMALEVEL |        // Use DMA level
                              DMAEN |           // Enable DMA3
                              DMAREQ;           // Trigger DMA
#else
                    // Enable transmit interrupt
                    HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxIE) |= UCTXIE;

                    // Preload the buffer for the first byte
                    HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxTXBUF) =
                        *sensorDataTx;
                    sensorDataTx++;
                    sensorDataTxSize--;
#endif
                }
                break;
            default: break;
            }
        }
#if defined(DEBUG_EEPROM)
        // Clear P1.0 for debugging purposes
        P1OUT &= ~BIT0;
#endif
        break;
    case USCI_SPI_UCTXIFG:                  // Vector 4: UXTXIFG
#ifndef USE_DMA
        if (spiOpCode == OPCODE_READ)
        {
            HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxTXBUF) =
                eeprom_getMemory(eepromReadIndexPointer++);
        }
        else
        {
            HWREG16(EEPROM_SPI_EUSCI_BASE + OFS_UCBxTXBUF) =
                *sensorDataTx;
            sensorDataTx++;
            sensorDataTxSize--;
        }
#endif
        break;
    default: break;
    }
}

//------------------------------------------------------------------------------
// Port 8 interrupt service routine.
// MSP-EXP430FR5994 uses P8.2 for CS and P8.3 for write protection bit.
// MSP-EXP430FR4133 uses P1.4 for CS and P1.3 for write protection bit.
//------------------------------------------------------------------------------
#pragma vector=PORTX_VECTOR
__interrupt void portX_isr_handler(void)
{
    switch(__even_in_range(PORT_IV_REG, PxIV_PxIFG7))
    {
    case PxIV_NONE:    break;               // Vector  0:  No interrupt
    case PxIV_PxIFG0:  break;               // Vector  2:  Px.0 IFG
    case PxIV_PxIFG1:  break;               // Vector  4:  Px.1 IFG
    case PxIV_PxIFG2:                       // Vector  6:  Px.2 IFG
#if defined(DEBUG_EEPROM)
        // Clear P1.1 for debugging purposes
        P1OUT |= BIT1;
#endif
        if(EEPROM_SLAVE_CSn_IES & EEPROM_SLAVE_CSn_PIN)
        {
            // Set the state to LPM0
            lpmState = 0;

            // Switch to LPM0 from LPM3 on starting the data transmission
            __bic_SR_register_on_exit(SCG1 | SCG0);

            // Enable the SPI module from reset state
            // and clear the interrupt flags
            eeprom_reEnableSpi();

            // Means a high-to-low transition occurred
            // Configure CS pin as low to high transition
            EEPROM_SLAVE_CSn_IES &= ~EEPROM_SLAVE_CSn_PIN;
        }
        else
        {
            // Means a low-to-high transition occurred

            // Set the state back to LPM3
            lpmState = 1;

            // Switch back to LPM3 from LPM0 on powering down
            __bis_SR_register_on_exit(SCG1 | SCG0);

            // Configure CS pin as high to low transition
            EEPROM_SLAVE_CSn_IES |= EEPROM_SLAVE_CSn_PIN;

            // Since the SPI RX interrupt has triggered twice during this period,
            // it needs to set the RX index pointer back by 2 counts
            eepromReadIndexPointer = eepromReadIndexPointer - 2;

            // Reset the SPI buffer data pointer
            eepromRxDataPointer = 0;

            // Clear the opcode flag
            spiOpCode = 0;

            // Clear the sensor protocol header tag
            sensorHeaderProtocol = 0;

#ifdef USE_DMA
            // Disable DMA and update pointers
            eeprom_disableDma();
#endif

            // Disable the SPI module
            eeprom_disableSpi();
        }
#if defined(DEBUG_EEPROM)
        // Clear P1.1 for debugging purposes
        P1OUT &= ~BIT1;
#endif
        break;
    case PxIV_PxIFG3:                       // Vector  8:  Px.3 IFG
        eeprom_writeProtectInterrupt();
        break;
    case PxIV_PxIFG4:                       // Vector  10: Px.4 IFG
#if defined(__MSP430FR2XX_4XX_FAMILY__)
#if defined(DEBUG_EEPROM)
        // Clear P1.1 for debugging purposes
        P1OUT |= BIT1;
#endif
        if(EEPROM_SLAVE_CSn_IES & EEPROM_SLAVE_CSn_PIN)
        {
            // Set the state to LPM0
            lpmState = 0;

            // Switch to LPM0 from LPM3 on starting the data transmission
            __bic_SR_register_on_exit(SCG1 | SCG0);

            // Enable the SPI module from reset state
            // and clear the interrupt flags
            eeprom_reEnableSpi();

            // Means a high-to-low transition occurred
            // Configure CS pin as low to high transition
            EEPROM_SLAVE_CSn_IES &= ~EEPROM_SLAVE_CSn_PIN;
        }
        else
        {
            // Means a low-to-high transition occurred

            // Set the state back to LPM3
            lpmState = 1;

            // Switch back to LPM3 from LPM0 on powering down
            __bis_SR_register_on_exit(SCG1 | SCG0);

            // Configure CS pin as high to low transition
            EEPROM_SLAVE_CSn_IES |= EEPROM_SLAVE_CSn_PIN;

            // Since the SPI RX interrupt has triggered twice during this period,
            // it needs to set the RX index pointer back by 2 counts
            eepromReadIndexPointer = eepromReadIndexPointer - 2;

            // Reset the SPI buffer data pointer
            eepromRxDataPointer = 0;

            // Clear the opcode flag
            spiOpCode = 0;

            // Clear the sensor protocol header tag
            sensorHeaderProtocol = 0;

#ifdef USE_DMA
            // Disable DMA and update pointers
            eeprom_disableDma();
#endif

            // Disable the SPI module
            eeprom_disableSpi();
        }
#if defined(DEBUG_EEPROM)
        // Clear P1.1 for debugging purposes
        P1OUT &= ~BIT1;
#endif
#endif
        break;
    case PxIV_PxIFG5:  break;               // Vector  12: Px.5 IFG
    case PxIV_PxIFG6:  break;               // Vector  14: Px.6 IFG
    case PxIV_PxIFG7:  break;               // Vector  14: Px.7 IFG
    default: break;
    }
}
