/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
  Description: This is the main project for the Slave in which it runs the
  	  	  	   EEPROM emulation with periodic background sampling of VCC and
  	  	  	   temperature (~500ms)

********************************************************************************/
#include <msp430.h>
#include <stdint.h>
#include <driverlib.h>
#include "eeprom.h"
#include "eeprom_definitions.h"
#include "sensing_proc.h"

// Data buffer storage from serial communication interface
uint8_t eepromRxData[EEPROM_ADDRESS_BYTE_SIZE];
uint8_t eepromRxDataPointer;

uint8_t lpmState;

#if defined(__IAR_SYSTEMS_ICC__)
__persistent float currentVcc;
__persistent uint8_t currentVccArray[sizeof(float)];
__persistent float currentTemperatureDegC;
__persistent uint8_t currentTemperatureDegCArray[sizeof(float)];
__persistent float currentTemperatureDegF;
__persistent uint8_t currentTemperatureDegFArray[sizeof(float)];
#elif defined(__TI_COMPILER_VERSION__)
#pragma PERSISTENT(currentVcc)
float currentVcc = 0.0;
#pragma PERSISTENT(currentVccArray)
uint8_t currentVccArray[sizeof(float)] = {0};
#pragma PERSISTENT(currentTemperatureDegC)
float currentTemperatureDegC = 0.0;
#pragma PERSISTENT(currentTemperatureDegCArray)
uint8_t currentTemperatureDegCArray[sizeof(float)] = {0};
#pragma PERSISTENT(currentTemperatureDegF)
float currentTemperatureDegF = 0.0;
#pragma PERSISTENT(currentTemperatureDegFArray)
uint8_t currentTemperatureDegFArray[sizeof(float)] = {0};
#endif

void init_gpio(void);

int main(void)
{
    // Stop watchdog timer
    WDT_A_hold(WDT_A_BASE);

    // Initializes the GPIO to the lowest power state
    init_gpio();

    // Disable the GPIO power-on default high-impedance mode to activate
    // previously configured port settings
    PMM_unlockLPM5();

    // Initializes system clocks to 16MHz with 1 wait-state
    FRCTL0 = FRCTLPW | NWAITS_1;
#if defined(__MSP430FR5XX_6XX_FAMILY__)
    CS_setDCOFreq(CS_DCORSEL_1, CS_DCOFSEL_4);

    //configure MCLK, SMCLK to be source by DCOCLK
    CS_initClockSignal(CS_SMCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);
    CS_initClockSignal(CS_MCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);
#else
    CS_initFLLSettle(16000, 487);

    //configure MCLK, SMCLK to be source by DCOCLK
    CS_initClockSignal(CS_SMCLK, CS_DCOCLKDIV_SELECT, CS_CLOCK_DIVIDER_1);
    CS_initClockSignal(CS_MCLK, CS_DCOCLKDIV_SELECT, CS_CLOCK_DIVIDER_1);

    // Disable the global FRAM protection for __persistent()
    SysCtl_enableFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_PROGRAM);
#endif

    // Initialize the received data pointers
    eepromRxDataPointer = 0;

    // Initializes the serial communication interface and necessary
    // pointers for the appliation.
    eeprom_initializeInterface();

    // Initialize the sensing functionality timer
    init_wakeUpTimer();

    // Initialize use LPM3 as the low power state
    lpmState = 1;

    while(1)
    {
        // Enter low power mode 3 and wait...
        if(lpmState)
        {
            __bis_SR_register(LPM3_bits | GIE);
        }
        else
        {
            // Stay back in LPM0 if it is in a middle of SPI or I2C transfer
            __bis_SR_register(LPM0_bits | GIE);
        }
        __no_operation();

        if(startSamplingData)
        {
            // Sample the data and store it in memory.
            // Perform this function only when no serial communication is ongoing
            sampleData();

            // Set flag that it is done sampling data
            startSamplingData = false;
        }
    }
}

// Initialize all GPIOs to lowest power state
void init_gpio(void)
{
#if defined(__MSP430_HAS_PORTA_R__)
    // Set PA as output direction
    GPIO_setAsOutputPin(GPIO_PORT_PA, 0xFFFF);

    // Set PA as output low
    GPIO_setOutputLowOnPin(GPIO_PORT_PA, 0xFFFF);
#endif

#if defined(__MSP430_HAS_PORTB_R__)
    // Set PB as output direction
    GPIO_setAsOutputPin(GPIO_PORT_PB, 0xFFFF);

    // Set PB as output low
    GPIO_setOutputLowOnPin(GPIO_PORT_PB, 0xFFFE);

    // Set 4.0 as output high for SD-Card CS pin needs to be high
    GPIO_setOutputHighOnPin(GPIO_PORT_P4, GPIO_PIN0);
#endif

#if defined(__MSP430_HAS_PORTC_R__)
    // Set PC as output direction
    GPIO_setAsOutputPin(GPIO_PORT_PC, 0xFFFF);

    // Set PC as output low
    GPIO_setOutputLowOnPin(GPIO_PORT_PC, 0xFFFF);
#endif

#if defined(__MSP430_HAS_PORTD_R__)
    // Set PD as output direction
    GPIO_setAsOutputPin(GPIO_PORT_PD, 0xFFFF);

    // Set PD as output low
    GPIO_setOutputLowOnPin(GPIO_PORT_PD, 0xFFFF);
#endif

#if defined(__MSP430_HAS_PORTE_R__)
    // Set PE as output direction
    GPIO_setAsOutputPin(GPIO_PORT_PE, 0xFFFF);

    // Set PE as output low
    GPIO_setOutputLowOnPin(GPIO_PORT_PE, 0xFFFF);
#endif

#if defined(__MSP430_HAS_PORTF_R__)
    // Set PF as output direction
    GPIO_setAsOutputPin(GPIO_PORT_PF, 0xFFFF);

    // Set PF as output low
    GPIO_setOutputLowOnPin(GPIO_PORT_PF, 0xFFFF);
#endif

#if defined(__MSP430_HAS_PORTJ_R__)
    // Set PJ as output direction
    GPIO_setAsOutputPin(GPIO_PORT_PJ, 0xFFFF);

    // Set PJ as output low
    GPIO_setOutputLowOnPin(GPIO_PORT_PJ, 0xFFFF);
#endif
}
