/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#include <msp430.h>
#include <stdint.h>
#include <string.h>
#include <driverlib.h>
#include "sensing_proc.h"
#include "eeprom_definitions.h"

// Temperature Sensor Calibration-30 C. See device datasheet for
// TLV table memory mapping
// This is for MSP430FR5xx_MSP430FR6xx family
#define CALADC12_12V_30C  *((unsigned int *)0x1A1A)
// Temperature Sensor Calibration-85 C
#define CALADC12_12V_85C  *((unsigned int *)0x1A1C)

// Temperature Sensor Calibration-30 C. See device datasheet for
// TLV table memory mapping
// This is for MSP430FR2xx_MSP430FR4xx family
#define CALADC_15V_30C  *((unsigned int *)0x1A1A)
// Temperature Sensor Calibration-85 C
#define CALADC_15V_85C  *((unsigned int *)0x1A1C)

// Global flag that gets set periodically in a timer indicating when
// main should start sampling new sensor data
bool startSamplingData = false;

void init_wakeUpTimer(void)
{
    // Timer_A2 PWM Configuration Parameter - wakes-up the processor periodically
    // to sample the ADC at approximately 500Hz
    Timer_A_initUpModeParam initWakeUpParam =
    {
        TIMER_A_CLOCKSOURCE_ACLK,               // ACLK Clock Source
        TIMER_A_CLOCKSOURCE_DIVIDER_1,          // ACLK/1 = 32768Hz
        (32768 / VCC_SAMPLING_FREQUENCY) - 1,   // period
        TIMER_A_TAIE_INTERRUPT_DISABLE,         // Disable Timer interrupt
        TIMER_A_CCIE_CCR0_INTERRUPT_ENABLE,     // Enable capture compare interrupt
        TIMER_A_DO_CLEAR,                       // Clear the initial count
        true                                    // Start the timer
    };

    Timer_A_initUpMode(TIMER_A1_BASE, &initWakeUpParam);
}

void sampleData(void)
{
    uint16_t vccValue;
    uint16_t tempValue;

#if defined(__MSP430FR5XX_6XX_FAMILY__)
    //Select internal ref = 2.0V and turn on Reference Voltage
    while(Ref_A_isRefGenBusy(REF_A_BASE))
    {
        ;
    }
    Ref_A_setReferenceVoltage(REF_A_BASE, REF_A_VREF2_0V);
    Ref_A_enableReferenceVoltage(REF_A_BASE);

    // ADC configuration parameter
    ADC12_B_initParam initAdcParam =
    {
        ADC12_B_SAMPLEHOLDSOURCE_SC,            // Manual sampling trigger
        ADC12_B_CLOCKSOURCE_ADC12OSC,           // Use ADC12OSC
        ADC12_B_CLOCKDIVIDER_1,                 // default divider
        ADC12_B_CLOCKPREDIVIDER__1,             // Default pre divider
        ADC12_B_BATTMAP                         // Perform battery measurement
    };

    //Initialize the ADC12B Module
    ADC12_B_init(ADC12_B_BASE, (ADC12_B_initParam *) &initAdcParam);
    ADC12_B_setupSamplingTimer(ADC12_B_BASE,
                               ADC12_B_CYCLEHOLD_16_CYCLES,
                               ADC12_B_CYCLEHOLD_4_CYCLES,
                               ADC12_B_MULTIPLESAMPLESDISABLE);

    ADC12_B_configureMemoryParam configureAdcMemoryParam =
    {
        ADC12_B_MEMORY_0,                       // Use ADC12MEM0 for ADC result
        ADC12_B_INPUT_BATMAP,                   // Set battery measurement as adc channel
        ADC12_B_VREFPOS_INTBUF_VREFNEG_VSS,     // default divider
        ADC12_B_ENDOFSEQUENCE,                  // Only 1 conversion to do
        ADC12_B_WINDOW_COMPARATOR_DISABLE,      // Enable window comparator
        ADC12_B_DIFFERENTIAL_MODE_DISABLE       // Disable differential mode
    };

    // Configure MEM0 to monitor Vcc/2 and window comparator to detect when Vcc
    // drops below VCC_THRESHOLD.
    ADC12_B_configureMemory(ADC12_B_BASE,
                            (ADC12_B_configureMemoryParam *) &configureAdcMemoryParam);

    //Enable window comparator high interrupt first to make sure we are above
    // threshold at startup
    ADC12_B_clearInterrupt(ADC12_B_BASE,
                           0,
                           ADC12_B_IFG0);

    // Enable the ADC12
    ADC12_B_enable(ADC12_B_BASE);

    // Check if reference voltage has stabilized before starting the sample
    while(Ref_A_isVariableReferenceVoltageOutputReady(REF_A_BASE) ==
          REF_A_NOTREADY)
    {
        ;
    }

    // Start conversion
    ADC12_B_startConversion(ADC12_B_BASE,
                            ADC12_B_MEMORY_0,
                            ADC12_B_SINGLECHANNEL);

    // Poll till conversion is done
    while(!ADC12_B_getInterruptStatus(ADC12_B_BASE,
                                      0,
                                      ADC12_B_IFG0))
    {
        ;
    }

    // Get results
    vccValue = ADC12_B_getResults(ADC12_B_BASE, ADC12_B_MEMORY_0);

    currentVcc = ((float) vccValue / 1024);
    memcpy((char *) currentVccArray, (&currentVcc), sizeof(float));

    // Disable the ADC and now measure the temperature sensors
    ADC12_B_disable(ADC12_B_BASE);

    // Change internal ref = 1.2V
    while(Ref_A_isRefGenBusy(REF_A_BASE))
    {
        ;
    }
    Ref_A_setReferenceVoltage(REF_A_BASE, REF_A_VREF1_2V);

    // Change ADC configuration now to perform temperature reading
    initAdcParam.internalChannelMap = ADC12_B_TEMPSENSEMAP;
    configureAdcMemoryParam.inputSourceSelect = ADC12_B_INPUT_TCMAP;

    ADC12_B_init(ADC12_B_BASE, (ADC12_B_initParam *) &initAdcParam);
    ADC12_B_setupSamplingTimer(ADC12_B_BASE,
                               ADC12_B_CYCLEHOLD_256_CYCLES,
                               ADC12_B_CYCLEHOLD_256_CYCLES,
                               ADC12_B_MULTIPLESAMPLESDISABLE);
    ADC12_B_configureMemory(ADC12_B_BASE,
                            (ADC12_B_configureMemoryParam *) &configureAdcMemoryParam);

    ADC12_B_clearInterrupt(ADC12_B_BASE,
                           0,
                           ADC12_B_IFG0);

    // Enable the ADC12
    ADC12_B_enable(ADC12_B_BASE);

    // Start conversion
    ADC12_B_startConversion(ADC12_B_BASE,
                            ADC12_B_MEMORY_0,
                            ADC12_B_SINGLECHANNEL);

    // Poll till conversion is done
    while(!ADC12_B_getInterruptStatus(ADC12_B_BASE,
                                      0,
                                      ADC12_B_IFG0))
    {
        ;
    }
    // Get results
    tempValue = ADC12_B_getResults(ADC12_B_BASE, ADC12_B_MEMORY_0);

    // Disable reference to conserve power
    Ref_A_disableReferenceVoltage(REF_A_BASE);

    currentTemperatureDegC =
        (float)(((long)tempValue - CALADC12_12V_30C) * (85 - 30)) /
        (CALADC12_12V_85C - CALADC12_12V_30C) + 30.0f;
    memcpy((char *) currentTemperatureDegCArray, (&currentTemperatureDegC),
           sizeof(float));
#else
    // Disable the ADC module first to reconfigure
    ADC_disableConversions(ADC_BASE, ADC_PREEMPTCONVERSION);

    //Initialize the ADC Module
    /*
     * Base Address for the ADC Module
     * Use Timer trigger 1 as sample/hold signal to start conversion
     * USE MODOSC 5MHZ Digital Oscillator as clock source
     * Use default clock divider of 1
     */
    ADC_init(ADC_BASE,
             ADC_SAMPLEHOLDSOURCE_SC,
             ADC_CLOCKSOURCE_ADCOSC,
             ADC_CLOCKDIVIDER_1);

    ADC_enable(ADC_BASE);
    // Measure VCC first
    //Configure Memory Buffer
    /*
     * Base Address for the ADC Module
     * Use input A12 Temp Sensor
     * Use positive reference of Internally generated Vref
     * Use negative reference of AVss
     */
    ADC_configureMemory(ADC_BASE,
                        ADC_INPUT_REFVOLTAGE,
                        ADC_VREFPOS_AVCC,
                        ADC_VREFNEG_AVSS);

    ADC_setupSamplingTimer(ADC_BASE,
                           ADC_CYCLEHOLD_16_CYCLES,
                           ADC_MULTIPLESAMPLESDISABLE);

    ADC_clearInterrupt(ADC_BASE,
                       ADC_COMPLETED_INTERRUPT_FLAG);

    ADC_startConversion(ADC_BASE,
                        ADC_REPEATED_SINGLECHANNEL);

    // Poll till conversion is done
    while(!ADC_getInterruptStatus(ADC_BASE,
                                  ADC_COMPLETED_INTERRUPT_FLAG))
    {
        ;
    }

    // Get results
    vccValue = ADC_getResults(ADC_BASE);
    currentVcc = (1023 * 1.5) / ((float) vccValue);
    memcpy((char *) currentVccArray, (&currentVcc), sizeof(float));

    // Disable the ADC module first to reconfigure
    ADC_disableConversions(ADC_BASE, ADC_PREEMPTCONVERSION);

    // Measure the temperature now
    // Enable internal reference and temperature sensor
    PMM_enableInternalReference();
    PMM_enableTempSensor();

    //Configure Memory Buffer
    /*
     * Base Address for the ADC Module
     * Use input A12 Temp Sensor
     * Use positive reference of Internally generated Vref
     * Use negative reference of AVss
     */
    ADC_configureMemory(ADC_BASE,
                        ADC_INPUT_TEMPSENSOR,
                        ADC_VREFPOS_INT,
                        ADC_VREFNEG_AVSS);

    ADC_setupSamplingTimer(ADC_BASE,
                           ADC_CYCLEHOLD_256_CYCLES,
                           ADC_MULTIPLESAMPLESDISABLE);

    ADC_clearInterrupt(ADC_BASE,
                       ADC_COMPLETED_INTERRUPT_FLAG);

    // Delay for reference settling
    __delay_cycles(30000);

    ADC_startConversion(ADC_BASE,
                        ADC_REPEATED_SINGLECHANNEL);

    // Poll till conversion is done
    while(!ADC_getInterruptStatus(ADC_BASE,
                                  ADC_COMPLETED_INTERRUPT_FLAG))
    {
        ;
    }

    // Get results
    tempValue = ADC_getResults(ADC_BASE);
    currentTemperatureDegC =
        (float)(((long)tempValue - CALADC_15V_30C) * (85 - 30)) /
        ((CALADC_15V_85C - CALADC_15V_30C)) + 30.0f;

    memcpy((char *) currentTemperatureDegCArray, (&currentTemperatureDegC),
           sizeof(float));

    // Disable the ADC module first to reconfigure
    ADC_disableConversions(ADC_BASE, ADC_PREEMPTCONVERSION);
#endif

    // Temperature in Fahrenheit Tf = (9/5)*Tc + 32
    currentTemperatureDegF = currentTemperatureDegC * 9.0f / 5.0f + 32.0f;
    memcpy((char *) currentTemperatureDegFArray, (&currentTemperatureDegF),
           sizeof(float));
}

//******************************************************************************
//
//This is the Timer A2 interrupt vector service routine.
//
//******************************************************************************
#pragma vector=TIMER1_A0_VECTOR
__interrupt void timer_a1_isr(void)
{
    // Set the flag indicating in the main that it should start sampling data
    startSamplingData = true;

    // Wake-up the processor from low power mode
    __bic_SR_register_on_exit(LPM4_bits);
}
